@@ -1,19 +1,17 @@
-#copyright ReportLab Inc. 2000-2001
+#Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/graphics/charts/linecharts.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/graphics/charts/linecharts.py,v 1.44 2003/11/29 17:13:01 rgbecker Exp $
+#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/charts/linecharts.py
 """
 This modules defines a very preliminary Line Chart example.
 """
-__version__=''' $Id: linecharts.py,v 1.44 2003/11/29 17:13:01 rgbecker Exp $ '''
+__version__=''' $Id: linecharts.py 2996 2006-10-31 13:23:02Z rgbecker $ '''
 
 import string
-from types import FunctionType, StringType
 
 from reportlab.lib import colors
 from reportlab.lib.validators import isNumber, isColor, isColorOrNone, isListOfStrings, \
                                     isListOfStringsOrNone, SequenceOf, isBoolean, NoneOr, \
-                                    isListOfNumbersOrNone
+                                    isListOfNumbersOrNone, isStringOrNone
 from reportlab.lib.attrmap import *
 from reportlab.lib.formatters import Formatter
 from reportlab.graphics.widgetbase import Widget, TypedPropertyCollection, PropHolder
@@ -30,9 +28,48 @@ class LineChartProperties(PropHolder):
         strokeColor = AttrMapValue(isColorOrNone, desc='Color of a line.'),
         strokeDashArray = AttrMapValue(isListOfNumbersOrNone, desc='Dash array of a line.'),
         symbol = AttrMapValue(NoneOr(isSymbol), desc='Widget placed at data points.'),
+        shader = AttrMapValue(None, desc='Shader Class.'),
+        filler = AttrMapValue(None, desc='Filler Class.'),
+        name = AttrMapValue(isStringOrNone, desc='Name of the line.'),
         )
 
-class LineChart(PlotArea):
+class AbstractLineChart(PlotArea):
+
+    def makeSwatchSample(self,rowNo, x, y, width, height):
+        baseStyle = self.lines
+        styleIdx = rowNo % len(baseStyle)
+        style = baseStyle[styleIdx]
+        color = style.strokeColor
+        y = y+height/2.
+        if self.joinedLines:
+            dash = getattr(style, 'strokeDashArray', getattr(baseStyle,'strokeDashArray',None))
+            strokeWidth= getattr(style, 'strokeWidth', getattr(style, 'strokeWidth',None))
+            L = Line(x,y,x+width,y,strokeColor=color,strokeLineCap=0)
+            if strokeWidth: L.strokeWidth = strokeWidth
+            if dash: L.strokeDashArray = dash
+        else:
+            L = None
+
+        if hasattr(style, 'symbol'):
+            S = style.symbol
+        elif hasattr(baseStyle, 'symbol'):
+            S = baseStyle.symbol
+        else:
+            S = None
+
+        if S: S = uSymbol2Symbol(S,x+width/2.,y,color)
+        if S and L:
+            g = Group()
+            g.add(L)
+            g.add(S)
+            return g
+        return S or L
+
+    def getSeriesName(self,i,default=None):
+        '''return series name i or default'''
+        return getattr(self.lines[i],'name',default)
+
+class LineChart(AbstractLineChart):
     pass
 
 # This is conceptually similar to the VerticalBarChart.
@@ -205,15 +242,15 @@ class HorizontalLineChart(LineChart):
 
         if labelFmt is None:
             labelText = None
-        elif type(labelFmt) is StringType:
+        elif type(labelFmt) is str:
             if labelFmt == 'values':
                 labelText = self.lineLabelArray[rowNo][colNo]
             else:
                 labelText = labelFmt % labelValue
-        elif type(labelFmt) is FunctionType:
-            labelText = labelFmt(labelValue)
         elif isinstance(labelFmt, Formatter):
             labelText = labelFmt(labelValue)
+        elif callable(labelFmt):
+            labelText = labelFmt(labelValue)
         else:
             msg = "Unknown formatter type %s, expected string or function"
             raise Exception, msg % labelFmt
@@ -330,12 +367,8 @@ class HorizontalLineChart(LineChart):
 
         g.add(cA)
         g.add(vA)
-        vA.gridStart = cA._x
-        vA.gridEnd = cA._x+cA._length
-        cA.gridStart = vA._y
-        cA.gridEnd = vA._y+vA._length
-        cA.makeGrid(g,parent=self)
-        vA.makeGrid(g,parent=self)
+        cA.makeGrid(g,parent=self,dim=vA.getGridDims)
+        vA.makeGrid(g,parent=self,dim=cA.getGridDims)
         g.add(self.makeLines())
         for a in getattr(self,'annotations',()): g.add(a(self,cA.scale,vA.scale))
         return g
