@@ -1,7 +1,6 @@
-#copyright ReportLab Inc. 2000-2001
+#Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/graphics/charts/piecharts.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/graphics/charts/piecharts.py,v 1.39 2004/01/20 22:50:31 andy_robinson Exp $
+#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/charts/piecharts.py
 # experimental pie chart script.  Two types of pie - one is a monolithic
 #widget with all top-level properties, the other delegates most stuff to
 #a wedges collection whic lets you customize the group or every individual
@@ -12,7 +11,7 @@
 This permits you to customize and pop out individual wedges;
 supports elliptical and circular pies.
 """
-__version__=''' $Id: piecharts.py,v 1.39 2004/01/20 22:50:31 andy_robinson Exp $ '''
+__version__=''' $Id: piecharts.py 3021 2007-01-22 14:33:59Z rgbecker $ '''
 
 import copy
 from math import sin, cos, pi
@@ -24,14 +23,17 @@ from reportlab.lib.validators import isColor, isNumber, isListOfNumbersOrNone,\
                                     isBoolean, isListOfColors, isNumberOrNone,\
                                     isNoneOrListOfNoneOrStrings, isTextAnchor,\
                                     isNoneOrListOfNoneOrNumbers, isBoxAnchor,\
-                                    isStringOrNone
+                                    isStringOrNone, NoneOr
+from reportlab.graphics.widgets.markers import uSymbol2Symbol, isSymbol
 from reportlab.lib.attrmap import *
 from reportlab.pdfgen.canvas import Canvas
-from reportlab.graphics.shapes import Group, Drawing, Ellipse, Wedge, String, STATE_DEFAULTS, ArcPath, Polygon
+from reportlab.graphics.shapes import Group, Drawing, Ellipse, Wedge, String, STATE_DEFAULTS, ArcPath, Polygon, Rect, PolyLine
 from reportlab.graphics.widgetbase import Widget, TypedPropertyCollection, PropHolder
+from reportlab.graphics.charts.areas import PlotArea
 from textlabels import Label
 
 _ANGLE2BOXANCHOR={0:'w', 45:'sw', 90:'s', 135:'se', 180:'e', 225:'ne', 270:'n', 315: 'nw', -45: 'nw'}
+_ANGLE2RBOXANCHOR={0:'e', 45:'ne', 90:'n', 135:'nw', 180:'w', 225:'sw', 270:'s', 315: 'se', -45: 'se'}
 class WedgeLabel(Label):
     def _checkDXY(self,ba):
         pass
@@ -41,7 +43,7 @@ class WedgeLabel(Label):
             da = (self._pmv - na) % 360
             if abs(da)>5:
                 na = na + (da>0 and 45 or -45)
-        ba = _ANGLE2BOXANCHOR[na]
+        ba = (getattr(self,'_anti',None) and _ANGLE2RBOXANCHOR or _ANGLE2BOXANCHOR)[na]
         self._checkDXY(ba)
         return ba
 
@@ -53,7 +55,6 @@ class WedgeProperties(PropHolder):
     angles.  It can format a genuine Wedge object for you with its
     format method.
     """
-
     _attrMap = AttrMap(
         strokeWidth = AttrMapValue(isNumber),
         fillColor = AttrMapValue(isColorOrNone),
@@ -84,6 +85,13 @@ class WedgeProperties(PropHolder):
         label_leftPadding = AttrMapValue(isNumber,'padding at left of box'),
         label_rightPadding = AttrMapValue(isNumber,'padding at right of box'),
         label_bottomPadding = AttrMapValue(isNumber,'padding at bottom of box'),
+        label_pointer_strokeColor = AttrMapValue(isColorOrNone,desc='Color of indicator line'),
+        label_pointer_strokeWidth = AttrMapValue(isNumber,desc='StrokeWidth of indicator line'),
+        label_pointer_elbowLength = AttrMapValue(isNumber,desc='length of final indicator line segment'),
+        label_pointer_edgePad = AttrMapValue(isNumber,desc='pad between pointer label and box'),
+        label_pointer_piePad = AttrMapValue(isNumber,desc='pad between pointer label and pie'),
+        swatchMarker = AttrMapValue(NoneOr(isSymbol), desc="None or makeMarker('Diamond') ..."),
+        visible = AttrMapValue(isBoolean,'set to false to skip displaying'),
         )
 
     def __init__(self):
@@ -108,12 +116,19 @@ class WedgeProperties(PropHolder):
         self.label_leading =    self.label_width = self.label_maxWidth = self.label_height = None
         self.label_textAnchor = 'start'
         self.label_visible = 1
+        self.label_pointer_strokeColor = colors.black
+        self.label_pointer_strokeWidth = 0.5
+        self.label_pointer_elbowLength = 3
+        self.label_pointer_edgePad = 2
+        self.label_pointer_piePad = 3
+        self.visible = 1
 
 def _addWedgeLabel(self,text,add,angle,labelX,labelY,wedgeStyle,labelClass=WedgeLabel):
     # now draw a label
     if self.simpleLabels:
         theLabel = String(labelX, labelY, text)
         theLabel.textAnchor = "middle"
+        theLabel._pmv = angle
     else:
         theLabel = labelClass()
         theLabel._pmv = angle
@@ -154,38 +169,282 @@ def _fixLabels(labels,n):
         if i>0: labels = labels + ['']*i
     return labels
 
-class Pie(Widget):
-    _attrMap = AttrMap(
-        x = AttrMapValue(isNumber, desc='X position of the chart within its container.'),
-        y = AttrMapValue(isNumber, desc='Y position of the chart within its container.'),
-        width = AttrMapValue(isNumber, desc='width of pie bounding box. Need not be same as width.'),
-        height = AttrMapValue(isNumber, desc='height of pie bounding box.  Need not be same as height.'),
+class AbstractPieChart(PlotArea):
+
+    def makeSwatchSample(self, rowNo, x, y, width, height):
+        baseStyle = self.slices
+        styleIdx = rowNo % len(baseStyle)
+        style = baseStyle[styleIdx]
+        strokeColor = getattr(style, 'strokeColor', getattr(baseStyle,'strokeColor',None))
+        fillColor = getattr(style, 'fillColor', getattr(baseStyle,'fillColor',None))
+        strokeDashArray = getattr(style, 'strokeDashArray', getattr(baseStyle,'strokeDashArray',None))
+        strokeWidth = getattr(style, 'strokeWidth', getattr(baseStyle, 'strokeWidth',None))
+        swatchMarker = getattr(style, 'swatchMarker', getattr(baseStyle, 'swatchMarker',None))
+        if swatchMarker:
+            return uSymbol2Symbol(swatchMarker,x+width/2.,y+height/2.,fillColor)
+        return Rect(x,y,width,height,strokeWidth=strokeWidth,strokeColor=strokeColor,
+                    strokeDashArray=strokeDashArray,fillColor=fillColor)
+
+    def getSeriesName(self,i,default=None):
+        '''return series name i or default'''
+        try:
+            text = str(self.labels[i])
+        except:
+            text = default
+        if not self.simpleLabels:
+            _text = getattr(self.slices[i],'label_text','')
+            if _text is not None: text = _text
+        return text
+
+def boundsOverlap(P,Q):
+    return not(P[0]>Q[2]-1e-2 or Q[0]>P[2]-1e-2 or P[1]>Q[3]-1e-2 or Q[1]>P[3]-1e-2)
+
+def _findOverlapRun(B,i,wrap):
+    '''find overlap run containing B[i]'''
+    n = len(B)
+    R = [i]
+    while 1:
+        i = R[-1]
+        j = (i+1)%n
+        if j in R or not boundsOverlap(B[i],B[j]): break
+        R.append(j)
+    while 1:
+        i = R[0]
+        j = (i-1)%n
+        if j in R or not boundsOverlap(B[i],B[j]): break
+        R.insert(0,j)
+    return R
+
+def findOverlapRun(B,wrap=1):
+    '''determine a set of overlaps in bounding boxes B or return None'''
+    n = len(B)
+    if n>1:
+        for i in xrange(n-1):
+            R = _findOverlapRun(B,i,wrap)
+            if len(R)>1: return R
+    return None
+
+def fixLabelOverlaps(L):
+    nL = len(L)
+    if nL<2: return
+    B = [l._origdata['bounds'] for l in L]
+    OK = 1
+    RP = []
+    iter = 0
+    mult = 1.
+
+    while iter<30:
+        R = findOverlapRun(B)
+        if not R: break
+        nR = len(R)
+        if nR==nL: break
+        if not [r for r in RP if r in R]:
+            mult = 1.0
+        da = 0
+        r0 = R[0]
+        rL = R[-1]
+        bi = B[r0]
+        taa = aa = _360(L[r0]._pmv)
+        for r in R[1:]:
+            b = B[r]
+            da = max(da,min(b[3]-bi[1],bi[3]-b[1]))
+            bi = b
+            aa += L[r]._pmv
+        aa = aa/float(nR)
+        utaa = abs(L[rL]._pmv-taa)
+        ntaa = _360(utaa)
+        da *= mult*(nR-1)/ntaa
+
+        for r in R:
+            l = L[r]
+            orig = l._origdata
+            angle = l._pmv = _360(l._pmv+da*(_360(l._pmv)-aa))
+            rad = angle/_180_pi
+            l.x = orig['cx'] + orig['rx']*cos(rad)
+            l.y = orig['cy'] + orig['ry']*sin(rad)
+            B[r] = l.getBounds()
+        RP = R
+        mult *= 1.05
+        iter += 1
+
+def intervalIntersection(A,B):
+    x,y = max(min(A),min(B)),min(max(A),max(B))
+    if x>=y: return None
+    return x,y
+
+def _makeSideArcDefs(sa,direction):
+    sa %= 360
+    if 90<=sa<270:
+        if direction=='clockwise':
+            a = (0,90,sa),(1,-90,90),(0,-360+sa,-90)
+        else:
+            a = (0,sa,270),(1,270,450),(0,450,360+sa)
+    else:
+        offs = sa>=270 and 360 or 0
+        if direction=='clockwise':
+            a = (1,offs-90,sa),(0,offs-270,offs-90),(1,-360+sa,offs-270)
+        else:
+            a = (1,sa,offs+90),(0,offs+90,offs+270),(1,offs+270,360+sa)
+    return tuple([a for a in a if a[1]<a[2]])
+
+def _findLargestArc(xArcs,side):
+    a = [a[1] for a in xArcs if a[0]==side and a[1] is not None]
+    if not a: return None
+    if len(a)>1: a.sort(lambda x,y: cmp(y[1]-y[0],x[1]-x[0]))
+    return a[0]
+
+def _fPLSide(l,width,side=None):
+    data = l._origdata
+    if side is None:
+        li = data['li']
+        ri = data['ri']
+        if li is None:
+            side = 1
+            i = ri
+        elif ri is None:
+            side = 0
+            i = li
+        elif li[1]-li[0]>ri[1]-ri[0]:
+            side = 0
+            i = li
+        else:
+            side = 1
+            i = ri
+    w = data['width']
+    edgePad = data['edgePad']
+    if not side:    #on left
+        l._pmv = 180
+        l.x = edgePad+w
+        i = data['li']
+    else:
+        l._pmv = 0
+        l.x = width - w - edgePad
+        i = data['ri']
+    mid = data['mid'] = (i[0]+i[1])*0.5
+    data['smid'] = sin(mid/_180_pi)
+    data['cmid'] = cos(mid/_180_pi)
+    data['side'] = side
+    return side,w
+
+def _fPLCF(a,b):
+    return cmp(b._origdata['smid'],a._origdata['smid'])
+
+def _arcCF(a,b):
+    return cmp(a[1],b[1])
+
+def _fixPointerLabels(n,L,x,y,width,height,side=None):
+    LR = [],[]
+    mlr = [0,0]
+    for l in L:
+        i,w = _fPLSide(l,width,side)
+        LR[i].append(l)
+        mlr[i] = max(w,mlr[i])
+    mul = 1
+    G = n*[None]
+    mel = 0
+    hh = height*0.5
+    yhh = y+hh
+    m = max(mlr)
+    for i in (0,1):
+        T = LR[i]
+        if T:
+            B = []
+            aB = B.append
+            S = []
+            aS = S.append
+            T.sort(_fPLCF)
+            p = 0
+            yh = y+height
+            for l in T:
+                data = l._origdata
+                inc = x+mul*(m-data['width'])
+                l.x += inc
+                G[data['index']] = l
+                ly = yhh+data['smid']*hh
+                b = data['bounds']
+                b2 = (b[3]-b[1])*0.5
+                if ly+b2>yh: ly = yh-b2
+                if ly-b2<y: ly = y+b2
+                data['bounds'] = b = (b[0],ly-b2,b[2],ly+b2)
+                aB(b)
+                l.y = ly
+                aS(max(0,yh-ly-b2))
+                yh = ly-b2
+                p = max(p,data['edgePad']+data['piePad'])
+                mel = max(mel,abs(data['smid']*(hh+data['elbowLength']))-hh)
+            aS(yh-y)
+
+            iter = 0
+            nT = len(T)
+            while iter<30:
+                R = findOverlapRun(B,wrap=0)
+                if not R: break
+                nR = len(R)
+                if nR==nT: break
+                j0 = R[0]
+                j1 = R[-1]
+                jl = j1+1
+                sAbove = sum(S[:j0+1])
+                sFree = sAbove+sum(S[jl:])
+                sNeed = sum([b[3]-b[1] for b in B[j0:jl]])+jl-j0-(B[j0][3]-B[j1][1])
+                if sNeed>sFree: break
+                yh = B[j0][3]+sAbove*sNeed/sFree
+                for r in R:
+                    l = T[r]
+                    data = l._origdata
+                    b = data['bounds']
+                    b2 = (b[3]-b[1])*0.5
+                    yh -= 0.5
+                    ly = l.y = yh-b2
+                    B[r] = data['bounds'] = (b[0],ly-b2,b[2],yh)
+                    yh = ly - b2 - 0.5
+            mlr[i] = m+p
+        mul = -1
+    return G, mlr[0], mlr[1], mel
+
+class Pie(AbstractPieChart):
+    _attrMap = AttrMap(BASE=AbstractPieChart,
         data = AttrMapValue(isListOfNumbers, desc='list of numbers defining wedge sizes; need not sum to 1'),
         labels = AttrMapValue(isListOfStringsOrNone, desc="optional list of labels to use for each data point"),
         startAngle = AttrMapValue(isNumber, desc="angle of first slice; like the compass, 0 is due North"),
-        direction = AttrMapValue( OneOf('clockwise', 'anticlockwise'), desc="'clockwise' or 'anticlockwise'"),
+        direction = AttrMapValue(OneOf('clockwise', 'anticlockwise'), desc="'clockwise' or 'anticlockwise'"),
         slices = AttrMapValue(None, desc="collection of wedge descriptor objects"),
         simpleLabels = AttrMapValue(isBoolean, desc="If true(default) use String not super duper WedgeLabel"),
-        other_threshold = AttrMapValue(isNumber, desc='A value for doing thresh holding, not used yet.'),
+        other_threshold = AttrMapValue(isNumber, desc='A value for doing threshholding, not used yet.'),
+        checkLabelOverlap = AttrMapValue(isBoolean, desc="If true check and attempt to fix standard label overlaps(default off)"),
+        pointerLabelMode = AttrMapValue(OneOf(None,'LeftRight','LeftAndRight'), desc=""),
+        sameRadii = AttrMapValue(isBoolean, desc="If true make x/y radii the same(default off)"),
+        orderMode = AttrMapValue(OneOf('fixed','alternate')),
+        xradius = AttrMapValue(isNumberOrNone, desc="X direction Radius"),
+        yradius = AttrMapValue(isNumberOrNone, desc="Y direction Radius"),
         )
     other_threshold=None
 
-    def __init__(self):
+    def __init__(self,**kwd):
+        PlotArea.__init__(self)
         self.x = 0
         self.y = 0
         self.width = 100
         self.height = 100
-        self.data = [1]
+        self.data = [1,2.3,1.7,4.2]
         self.labels = None  # or list of strings
         self.startAngle = 90
         self.direction = "clockwise"
         self.simpleLabels = 1
+        self.checkLabelOverlap = 0
+        self.pointerLabelMode = None
+        self.sameRadii = False
+        self.orderMode = 'fixed'
+        self.xradius = self.yradius = None
 
         self.slices = TypedPropertyCollection(WedgeProperties)
         self.slices[0].fillColor = colors.darkcyan
         self.slices[1].fillColor = colors.blueviolet
         self.slices[2].fillColor = colors.blue
         self.slices[3].fillColor = colors.cyan
+        self.slices[4].fillColor = colors.pink
+        self.slices[5].fillColor = colors.magenta
+        self.slices[6].fillColor = colors.yellow
 
     def demo(self):
         d = Drawing(200, 100)
@@ -215,86 +474,235 @@ class Pie(Widget):
         d.add(pc)
         return d
 
+    def makePointerLabels(self,angles,plMode):
+        class PL:
+            def __init__(self,centerx,centery,xradius,yradius,data,lu=0,ru=0):
+                self.centerx = centerx
+                self.centery = centery
+                self.xradius = xradius
+                self.yradius = yradius
+                self.data = data
+                self.lu = lu
+                self.ru = ru
+
+        labelX = self.width-2
+        labelY = self.height
+        n = nr = nl = maxW = sumH = 0
+        styleCount = len(self.slices)
+        L=[]
+        L_add = L.append
+        refArcs = _makeSideArcDefs(self.startAngle,self.direction)
+        for i, A in angles:
+            if A[1] is None: continue
+            sn = self.getSeriesName(i,'')
+            if not sn: continue
+            style = self.slices[i%styleCount]
+            if not style.label_visible or not style.visible: continue
+            n += 1
+            _addWedgeLabel(self,sn,L_add,180,labelX,labelY,style,labelClass=WedgeLabel)
+            l = L[-1]
+            b = l.getBounds()
+            w = b[2]-b[0]
+            h = b[3]-b[1]
+            ri = [(a[0],intervalIntersection(A,(a[1],a[2]))) for a in refArcs]
+            li = _findLargestArc(ri,0)
+            ri = _findLargestArc(ri,1)
+            if li and ri:
+                if plMode=='LeftAndRight':
+                    if li[1]-li[0]<ri[1]-ri[0]:
+                        li = None
+                    else:
+                        ri = None
+                else:
+                    if li[1]-li[0]<0.02*(ri[1]-ri[0]):
+                        li = None
+                    elif (li[1]-li[0])*0.02>ri[1]-ri[0]:
+                        ri = None
+            if ri: nr += 1
+            if li: nl += 1
+            l._origdata = dict(bounds=b,width=w,height=h,li=li,ri=ri,index=i,edgePad=style.label_pointer_edgePad,piePad=style.label_pointer_piePad,elbowLength=style.label_pointer_elbowLength)
+            maxW = max(w,maxW)
+            sumH += h+2
+
+        if not n:   #we have no labels
+            xradius = self.width*0.5
+            yradius = self.height*0.5
+            centerx = self.x+xradius
+            centery = self.y+yradius
+            if self.xradius: xradius = self.xradius
+            if self.yradius: yradius = self.yradius
+            if self.sameRadii: xradius=yradius=min(xradius,yradius)
+            return PL(centerx,centery,xradius,yradius,[])
+
+        aonR = nr==n
+        if sumH<self.height and (aonR or nl==n):
+            side=int(aonR)
+        else:
+            side=None
+        G,lu,ru,mel = _fixPointerLabels(len(angles),L,self.x,self.y,self.width,self.height,side=side)
+        if plMode=='LeftAndRight':
+            lu = ru = max(lu,ru)
+        x0 = self.x+lu
+        x1 = self.x+self.width-ru
+        xradius = (x1-x0)*0.5
+        yradius = self.height*0.5-mel
+        centerx = x0+xradius
+        centery = self.y+yradius+mel
+        if self.xradius: xradius = self.xradius
+        if self.yradius: yradius = self.yradius
+        if self.sameRadii: xradius=yradius=min(xradius,yradius)
+        return PL(centerx,centery,xradius,yradius,G,lu,ru)
+
     def normalizeData(self):
         from operator import add
         data = self.data
         self._sum = sum = float(reduce(add,data,0))
         return abs(sum)>=1e-8 and map(lambda x,f=360./sum: f*x, data) or len(data)*[0]
 
+    def makeAngles(self):
+        startAngle = self.startAngle % 360
+        whichWay = self.direction == "clockwise" and -1 or 1
+        D = [a for a in enumerate(self.normalizeData())]
+        if self.orderMode=='alternate':
+            W = [a for a in D if abs(a[1])>=1e-5]
+            W.sort(_arcCF)
+            T = [[],[]]
+            i = 0
+            while W:
+                if i<2:
+                    a = W.pop(0)
+                else:
+                    a = W.pop(-1)
+                T[i%2].append(a)
+                i += 1
+                i %= 4
+            T[1].reverse()
+            D = T[0]+T[1] + [a for a in D if abs(a[1])<1e-5]
+        A = []
+        a = A.append
+        for i, angle in D:
+            endAngle = (startAngle + (angle * whichWay))
+            if abs(angle)>=1e-5:
+                if startAngle >= endAngle:
+                    aa = endAngle,startAngle
+                else:
+                    aa = startAngle,endAngle
+            else:
+                aa = startAngle, None
+            startAngle = endAngle
+            a((i,aa))
+        return A
+
     def makeWedges(self):
-        # normalize slice data
-        normData = self.normalizeData()
-        n = len(normData)
+        angles = self.makeAngles()
+        n = len(angles)
         labels = _fixLabels(self.labels,n)
 
-        xradius = self.width/2.0
-        yradius = self.height/2.0
-        centerx = self.x + xradius
-        centery = self.y + yradius
-
-        if self.direction == "anticlockwise":
-            whichWay = 1
-        else:
-            whichWay = -1
-
-        g = Group()
-        i = 0
+        self._seriesCount = n
         styleCount = len(self.slices)
 
-        startAngle = self.startAngle #% 360
-        for angle in normData:
-            endAngle = (startAngle + (angle * whichWay)) #% 360
-            if abs(startAngle-endAngle)>=1e-5:
-                if startAngle < endAngle:
-                    a1 = startAngle
-                    a2 = endAngle
-                else:
-                    a1 = endAngle
-                    a2 = startAngle
+        plMode = self.pointerLabelMode
+        if plMode:
+            checkLabelOverlap = False
+            PL=self.makePointerLabels(angles,plMode)
+            xradius = PL.xradius
+            yradius = PL.yradius
+            centerx = PL.centerx
+            centery = PL.centery
+            PL_data = PL.data
+            gSN = lambda i: ''
+        else:
+            xradius = self.width*0.5
+            yradius = self.height*0.5
+            centerx = self.x + xradius
+            centery = self.y + yradius
+            if self.xradius: xradius = self.xradius
+            if self.yradius: yradius = self.yradius
+            if self.sameRadii: xradius=yradius=min(xradius,yradius)
+            checkLabelOverlap = self.checkLabelOverlap
+            gSN = lambda i: self.getSeriesName(i,'')
 
-                #if we didn't use %stylecount here we'd end up with the later wedges
-                #all having the default style
-                wedgeStyle = self.slices[i%styleCount]
+        g = Group()
+        g_add = g.add
+        if checkLabelOverlap:
+            L = []
+            L_add = L.append
+        else:
+            L_add = g_add
 
-                # is it a popout?
-                cx, cy = centerx, centery
-                if wedgeStyle.popout <> 0:
+        for i,(a1,a2) in angles:
+            if a2 is None: continue
+            #if we didn't use %stylecount here we'd end up with the later wedges
+            #all having the default style
+            wedgeStyle = self.slices[i%styleCount]
+            if not wedgeStyle.visible: continue
+
+            # is it a popout?
+            cx, cy = centerx, centery
+            text = gSN(i)
+            popout = wedgeStyle.popout
+            if text or popout:
+                averageAngle = (a1+a2)/2.0
+                aveAngleRadians = averageAngle/_180_pi
+                cosAA = cos(aveAngleRadians)
+                sinAA = sin(aveAngleRadians)
+                if popout:
                     # pop out the wedge
-                    averageAngle = (a1+a2)/2.0
-                    aveAngleRadians = averageAngle * pi/180.0
-                    popdistance = wedgeStyle.popout
-                    cx = centerx + popdistance * cos(aveAngleRadians)
-                    cy = centery + popdistance * sin(aveAngleRadians)
+                    cx = centerx + popout*cosAA
+                    cy = centery + popout*sinAA
 
-                if n > 1:
-                    theWedge = Wedge(cx, cy, xradius, a1, a2, yradius=yradius)
-                elif n==1:
-                    theWedge = Ellipse(cx, cy, xradius, yradius)
+            if n > 1:
+                theWedge = Wedge(cx, cy, xradius, a1, a2, yradius=yradius)
+            elif n==1:
+                theWedge = Ellipse(cx, cy, xradius, yradius)
 
-                theWedge.fillColor = wedgeStyle.fillColor
-                theWedge.strokeColor = wedgeStyle.strokeColor
-                theWedge.strokeWidth = wedgeStyle.strokeWidth
-                theWedge.strokeDashArray = wedgeStyle.strokeDashArray
+            theWedge.fillColor = wedgeStyle.fillColor
+            theWedge.strokeColor = wedgeStyle.strokeColor
+            theWedge.strokeWidth = wedgeStyle.strokeWidth
+            theWedge.strokeDashArray = wedgeStyle.strokeDashArray
 
-                g.add(theWedge)
-                text = labels[i]
+            g_add(theWedge)
+            if wedgeStyle.label_visible:
                 if text:
-                    averageAngle = (a1+a2)/2.0
-                    aveAngleRadians = averageAngle*pi/180.0
                     labelRadius = wedgeStyle.labelRadius
-                    labelX = cx + (0.5 * self.width * cos(aveAngleRadians) * labelRadius)
-                    labelY = cy + (0.5 * self.height * sin(aveAngleRadians) * labelRadius)
-                    _addWedgeLabel(self,text,g.add,averageAngle,labelX,labelY,wedgeStyle)
+                    rx = xradius*labelRadius
+                    ry = yradius*labelRadius
+                    labelX = cx + rx*cosAA
+                    labelY = cy + ry*sinAA
+                    _addWedgeLabel(self,text,L_add,averageAngle,labelX,labelY,wedgeStyle)
+                    if checkLabelOverlap:
+                        l = L[-1]
+                        l._origdata = { 'x': labelX, 'y':labelY, 'angle': averageAngle,
+                                        'rx': rx, 'ry':ry, 'cx':cx, 'cy':cy,
+                                        'bounds': l.getBounds(),
+                                        }
+                elif plMode and PL_data:
+                    l = PL_data[i]
+                    if l:
+                        data = l._origdata
+                        sinM = data['smid']
+                        cosM = data['cmid']
+                        lX = cx + xradius*cosM
+                        lY = cy + yradius*sinM
+                        lpel = wedgeStyle.label_pointer_elbowLength
+                        lXi = lX + lpel*cosM
+                        lYi = lY + lpel*sinM
+                        L_add(PolyLine((lX,lY,lXi,lYi,l.x,l.y),
+                                strokeWidth=wedgeStyle.label_pointer_strokeWidth,
+                                strokeColor=wedgeStyle.label_pointer_strokeColor))
+                        L_add(l)
 
-            startAngle = endAngle
-            i = i + 1
+        if checkLabelOverlap and L:
+            fixLabelOverlaps(L)
+            map(g_add,L)
 
         return g
 
     def draw(self):
-        g = Group()
-        g.add(self.makeWedges())
-        return g
+        G = self.makeBackground()
+        w = self.makeWedges()
+        if G: return Group(G,w)
+        return w
 
 class LegendedPie(Pie):
     """Pie with a two part legend (one editable with swatches, one hidden without swatches)."""
@@ -449,7 +857,7 @@ class LegendedPie(Pie):
         drawing.add(self.draw())
         return drawing
 
-from utils3d import _getShaded, _2rad, _360, _pi_2, _2pi
+from utils3d import _getShaded, _2rad, _360, _pi_2, _2pi, _180_pi
 class Wedge3dProperties(PropHolder):
     """This holds descriptive information about the wedges in a pie chart.
 
@@ -576,6 +984,8 @@ class Pie3d(Pie):
         self.slices[4].fillColor = colors.azure
         self.slices[5].fillColor = colors.crimson
         self.slices[6].fillColor = colors.darkviolet
+        self.checkLabelOverlap = 0
+        self.xradius = self.yradius = None
 
     def _fillSide(self,L,i,angle,strokeColor,strokeWidth,fillColor):
         rd = self.rad_dist(angle)
@@ -595,9 +1005,11 @@ class Pie3d(Pie):
         self._ydepth_3d = sin(a0)*self.depth_3d
         self._cx = self.x+self.width/2.0
         self._cy = self.y+(self.height - self._ydepth_3d)/2.0
-        radius = self._radius = self._cx-self.x
-        self._radiusx = radiusx = radius
-        self._radiusy = radiusy = (1.0 - self.perspective/100.0)*radius
+        radiusx = radiusy = self._cx-self.x
+        if self.xradius: radiusx = self.xradius
+        if self.yradius: radiusy = self.yradius
+        self._radiusx = radiusx
+        self._radiusy = radiusy = (1.0 - self.perspective/100.0)*radiusy
         data = self.normalizeData()
         sum = self._sum
 
@@ -607,7 +1019,7 @@ class Pie3d(Pie):
         OY = self.OY
         rad_dist = self.rad_dist
         _fillSide = self._fillSide
-        n = len(data)
+        self._seriesCount = n = len(data)
         _sl3d = self._sl3d = []
         g = Group()
         last = _360(self.startAngle)
@@ -618,7 +1030,6 @@ class Pie3d(Pie):
             last = angle0
             if a0>0: angle0, angle1 = angle1, angle0
             _sl3d.append(_SL3D(angle0,angle1))
-            #print '%d: %.2f %.2f --> %s' %(len(_sl3d)-1,angle0,angle1,_sl3d[-1])
 
         labels = _fixLabels(self.labels,n)
         a0 = _3d_angle
@@ -628,12 +1039,14 @@ class Pie3d(Pie):
         L = []
 
         class WedgeLabel3d(WedgeLabel):
+            _ydepth_3d = self._ydepth_3d
             def _checkDXY(self,ba):
                 if ba[0]=='n':
                     if not hasattr(self,'_ody'):
                         self._ody = self.dy
                         self.dy = -self._ody + self._ydepth_3d
-        WedgeLabel3d._ydepth_3d = self._ydepth_3d
+    
+        checkLabelOverlap = self.checkLabelOverlap
 
         for i in xrange(n):
             style = slices[i]
@@ -677,16 +1090,26 @@ class Pie3d(Pie):
                             strokeColor=strokeColor,strokeWidth=strokeWidth,fillColor=fillColor,strokeLineJoin=1))
 
             text = labels[i]
-            if text:
+            if style.label_visible and text:
                 rat = style.labelRadius
                 self._radiusx *= rat
                 self._radiusy *= rat
                 mid = sl.mid
-                _addWedgeLabel(self,text,L.append,mid,OX(i,mid,0),OY(i,mid,0),style,labelClass=WedgeLabel3d)
+                labelX = OX(i,mid,0)
+                labelY = OY(i,mid,0)
+                _addWedgeLabel(self,text,L.append,mid,labelX,labelY,style,labelClass=WedgeLabel3d)
+                if checkLabelOverlap:
+                    l = L[-1]
+                    l._origdata = { 'x': labelX, 'y':labelY, 'angle': mid,
+                                    'rx': self._radiusx, 'ry':self._radiusy, 'cx':CX(i,0), 'cy':CY(i,0),
+                                    'bounds': l.getBounds(),
+                                    }
                 self._radiusx = radiusx
                 self._radiusy = radiusy
 
         S.sort(lambda a,b: -cmp(a[0],b[0]))
+        if checkLabelOverlap and L:
+            fixLabelOverlaps(L)
         map(g.add,map(lambda x:x[1],S)+T+L)
         return g
 
