@@ -14,7 +14,7 @@ from operator import getitem
 from reportlab.pdfbase.pdfmetrics import stringWidth # for font info
 from reportlab.lib.utils import fp_str
 from reportlab.lib.colors import black
-from reportlab.graphics.renderbase import StateTracker, getStateDelta, Renderer
+from reportlab.graphics.renderbase import StateTracker, getStateDelta, Renderer, renderScaledDrawing
 from reportlab.graphics.shapes import STATE_DEFAULTS, Path, UserNode
 from reportlab.graphics.shapes import * # (only for test0)
 from reportlab import rl_config
@@ -41,8 +41,8 @@ def drawToString(d, showBoundary=rl_config.showBoundary):
     drawToFile(d, s, showBoundary=showBoundary)
     return s.getvalue()
 
-
 def drawToFile(d, fn, showBoundary=rl_config.showBoundary):
+    d = renderScaledDrawing(d)
     c = SVGCanvas((d.width, d.height))
     draw(d, c, 0, 0, showBoundary=showBoundary)
     c.save(fn)
@@ -50,9 +50,8 @@ def drawToFile(d, fn, showBoundary=rl_config.showBoundary):
 
 def draw(drawing, canvas, x=0, y=0, showBoundary=rl_config.showBoundary):
     """As it says."""
-
     r = _SVGRenderer()
-    r.draw(drawing, canvas, x, y, showBoundary=showBoundary)
+    r.draw(renderScaledDrawing(drawing), canvas, x, y, showBoundary=showBoundary)
 
 
 ### helper functions ###
@@ -387,6 +386,22 @@ class SVGCanvas:
 
             self.currGroup.appendChild(text)
 
+    def drawCentredString(self, s, x, y, angle=0,text_anchor='middle'):
+        if self.verbose: print "+++ SVGCanvas.drawCentredString"
+
+        if self._fillColor != None:
+            if not text_anchor in ['start', 'inherited']:
+                textLen = stringWidth(s,self._font,self._fontSize)
+                if text_anchor=='end':
+                    x -= textLen
+                elif text_anchor=='middle':
+                    x -= textLen/2.
+                else:
+                    raise ValueError, 'bad value for text_anchor ' + str(text_anchor)
+        self.drawString(x,y,text,angle=angle)
+
+    def drawRightString(self, text, x, y, angle=0):
+        self.drawCentredString(text,x,y,angle=angle,text_anchor='end')
 
     def comment(self, data):
         "Add a comment."
@@ -585,8 +600,11 @@ class SVGCanvas:
     def closePath(self):
         self.path = self.path + 'Z '
 
+    def saveState(self):
+        pass
 
-
+    def restoreState(self):
+        pass
 
 class _SVGRenderer(Renderer):
     """This draws onto an SVG document.
@@ -596,33 +614,6 @@ class _SVGRenderer(Renderer):
         self._tracker = StateTracker()
         self.verbose = 0
 
-
-    def draw(self, drawing, canvas, x, y, showBoundary=rl_config.showBoundary):
-        """This is the top level function, which draws the drawing at the given
-        location. The recursive part is handled by drawNode.
-        """
-
-        if self.verbose: print "### begin _SVGRenderer.draw"
-
-        self._canvas = canvas
-        canvas._drawing = self._drawing = drawing
-        try:
-            #bounding box
-            if showBoundary:
-                canvas.rect(x, y, drawing.width, drawing.height)
-
-            deltas = STATE_DEFAULTS.copy()
-            deltas['transform'] = [1,0,0,1,x,y]
-            self._tracker.push(deltas)
-            self.applyStateChanges(deltas, {})
-            self.drawNode(drawing)
-            self._tracker.pop()
-        finally:
-            del self._canvas, self._drawing, canvas._drawing
-
-        if self.verbose: print "### end _SVGRenderer.draw"
-
-
     def drawNode(self, node):
         """This is the recursive method called for each node in the tree.
         """
@@ -721,7 +712,7 @@ class _SVGRenderer(Renderer):
     def drawWedge(self, wedge):
         centerx, centery, radius, startangledegrees, endangledegrees = \
          wedge.centerx, wedge.centery, wedge.radius, wedge.startangledegrees, wedge.endangledegrees
-        yradius = wedge.yradius
+        yradius = wedge.yradius or wedge.radius
         (x1, y1) = (centerx-radius, centery-yradius)
         (x2, y2) = (centerx+radius, centery+yradius)
         extent = endangledegrees - startangledegrees
@@ -834,4 +825,4 @@ def test2():
 if __name__=='__main__':
     test0()
     test1()
-    test2()
\ No newline at end of file
+    test2()
