@@ -1,11 +1,10 @@
-#copyright ReportLab Inc. 2001
+#Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/graphics/shapes.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/graphics/shapes.py,v 1.100 2003/11/22 14:41:24 rgbecker Exp $
+#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/shapes.py
 """
 core of the graphics library - defines Drawing and Shapes
 """
-__version__=''' $Id: shapes.py,v 1.100 2003/11/22 14:41:24 rgbecker Exp $ '''
+__version__=''' $Id: shapes.py 3089 2007-05-23 11:45:58Z rgbecker $ '''
 
 import string, os, sys
 from math import pi, cos, sin, tan
@@ -13,7 +12,7 @@ from types import FloatType, IntType, ListType, TupleType, StringType, InstanceT
 from pprint import pprint
 
 from reportlab.platypus import Flowable
-from reportlab.rl_config import shapeChecking, verbose
+from reportlab.rl_config import shapeChecking, verbose, defaultGraphicsFontName, _unset_
 from reportlab.lib import logger
 from reportlab.lib import colors
 from reportlab.lib.validators import *
@@ -52,7 +51,7 @@ STATE_DEFAULTS = {   # sensible defaults for all
     #'fillOpacity': 1.0,  #100% - can be done later
 
     'fontSize': 10,
-    'fontName': 'Times-Roman',
+    'fontName': defaultGraphicsFontName,
     'textAnchor':  'start' # can be start, middle, end, inherited
     }
 
@@ -189,15 +188,11 @@ class _SetKeyWordArgs:
 
 def getRectsBounds(rectList):
     # filter out any None objects, e.g. empty groups
-    rectList2 = []
-    for elem in rectList:
-        if elem == (0,0,0,0):
-            print 'got an empty one'
-        if elem is not None:
-            rectList2.append(elem)
-    xMin, yMin, xMax, yMax = rectList2[0]
+    L = filter(lambda x: x is not None, rectList)
+    if not L: return None
 
-    for (x1, y1, x2, y2) in rectList2[1:]:
+    xMin, yMin, xMax, yMax = L[0]
+    for (x1, y1, x2, y2) in L[1:]:
         if x1 < xMin:
             xMin = x1
         if x2 > xMax:
@@ -477,7 +472,9 @@ class Group(Shape):
             b = []
             for elem in self.contents:
                 b.append(elem.getBounds())
-            (x1, y1, x2, y2) = getRectsBounds(b)
+            x1 = getRectsBounds(b)
+            if x1 is None: return None
+            x1, y1, x2, y2 = x1
             trans = self.transform
             corners = [[x1,y1], [x1, y2], [x2, y1], [x2,y2]]
             newCorners = []
@@ -492,7 +489,7 @@ class Group(Shape):
             return None
 
 def _addObjImport(obj,I,n=None):
-    '''add an import of obj's class to a dictionary of imports'''
+    '''add an import of obj's class to a dictionary of imports''' #'
     from inspect import getmodule
     c = obj.__class__
     m = getmodule(c).__name__
@@ -553,18 +550,35 @@ def _renderGroupPy(G,pfx,I,i=0,indent='\t\t'):
             s = s + '%s%s.add(%s)\n' % (indent,pfx,_repr(n,I))
     return s
 
+def _extraKW(self,pfx,**kw):
+    kw.update(self.__dict__)
+    R = {}
+    n = len(pfx)
+    for k in kw.keys():
+        if k.startswith(pfx):
+            R[k[n:]] = kw[k]
+    return R
+
 class Drawing(Group, Flowable):
     """Outermost container; the thing a renderer works on.
     This has no properties except a height, width and list
     of contents."""
 
+    _saveModes=(
+            'pdf','ps','eps','gif','png','jpg','jpeg','pct',
+            'pict','tiff','tif','py','bmp','svg','tiffp',
+            )
+
     _xtraAttrMap = AttrMap(
         width = AttrMapValue(isNumber,desc="Drawing width in points."),
         height = AttrMapValue(isNumber,desc="Drawing height in points."),
         canv = AttrMapValue(None),
         background = AttrMapValue(isValidChildOrNone,desc="Background widget for the drawing"),
-        hAlign = AttrMapValue(OneOf("LEFT", "RIGHT", "CENTER", "CENTRE"), desc="Alignment within parent document"),
-        vAlign = AttrMapValue(OneOf("TOP", "BOTTOM", "CENTER", "CENTRE"), desc="Alignment within parent document")
+        hAlign = AttrMapValue(OneOf("LEFT", "RIGHT", "CENTER", "CENTRE"), desc="Horizontal alignment within parent document"),
+        vAlign = AttrMapValue(OneOf("TOP", "BOTTOM", "CENTER", "CENTRE"), desc="Vertical alignment within parent document"),
+        #AR temporary hack to track back up.
+        #fontName = AttrMapValue(isStringOrNone),
+        renderScale = AttrMapValue(isNumber,desc="Global scaling for rendering"),
         )
 
     _attrMap = AttrMap(BASE=Group)
@@ -577,6 +591,7 @@ class Drawing(Group, Flowable):
         self.height = height
         self.hAlign = 'LEFT'
         self.vAlign = 'BOTTOM'
+        self.renderScale = 1.0
 
     def _renderPy(self):
         I = {'reportlab.graphics.shapes': ['_DrawingEditorMixin','Drawing','Group']}
@@ -592,14 +607,21 @@ class Drawing(Group, Flowable):
         s = s + '\n\nif __name__=="__main__": #NORUNTESTS\n\t%s().save(formats=[\'pdf\'],outDir=\'.\',fnRoot=None)\n' % n
         return s
 
-    def draw(self):
+    def draw(self,showBoundary=_unset_):
         """This is used by the Platypus framework to let the document
         draw itself in a story.  It is specific to PDF and should not
         be used directly."""
-
         import renderPDF
-        R = renderPDF._PDFRenderer()
-        R.draw(self, self.canv, 0, 0)
+        renderPDF.draw(self, self.canv, 0, 0, showBoundary=showBoundary)
+
+    def wrap(self, availWidth, availHeight):
+        width = self.width
+        height = self.height
+        renderScale = self.renderScale
+        if renderScale!=1.0:
+            width *= renderScale
+            height *= renderScale
+        return width, height
 
     def expandUserNodes(self):
         """Return a new drawing which only contains primitive shapes."""
@@ -608,17 +630,20 @@ class Drawing(Group, Flowable):
         obj.height = self.height
         return obj
 
-    def copy(self,obj):
+    def copy(self):
         """Returns a copy"""
-        return self._copy(Drawing(self.width, self.height))
+        return self._copy(self.__class__(self.width, self.height))
 
     def asGroup(self,*args,**kw):
         return self._copy(apply(Group,args,kw))
 
-    def save(self, formats=None, verbose=None, fnRoot=None, outDir=None, title=''):
+    def save(self, formats=None, verbose=None, fnRoot=None, outDir=None, title='', **kw):
         """Saves copies of self in desired location and formats.
+        Multiple formats can be supported in one call
 
-        Multiple formats can be supported in one call"""
+        the extra keywords can be of the form
+        _renderPM_dpi=96 (which passes dpi=96 to renderPM)
+        """
         from reportlab import rl_config
         ext = ''
         if not fnRoot:
@@ -630,59 +655,78 @@ class Drawing(Group, Flowable):
                 try:
                     fnRoot = fnRoot % getattr(self,'chartId',0)
                 except TypeError, err:
-                    if str(err) != 'not all arguments converted': raise
+                    #the exact error message changed from 2.2 to 2.3 so we need to
+                    #check a substring
+                    if str(err).find('not all arguments converted') < 0: raise
 
         if os.path.isabs(fnRoot):
             outDir, fnRoot = os.path.split(fnRoot)
         else:
             outDir = outDir or getattr(self,'outDir','.')
-        if not os.path.isabs(outDir): outDir = os.path.join(os.path.dirname(sys.argv[0]),outDir)
+        outDir = outDir.rstrip().rstrip(os.sep)
+        if not outDir: outDir = '.'
+        if not os.path.isabs(outDir): outDir = os.path.join(getattr(self,'_override_CWD',os.path.dirname(sys.argv[0])),outDir)
         if not os.path.isdir(outDir): os.makedirs(outDir)
         fnroot = os.path.normpath(os.path.join(outDir,fnRoot))
         plotMode = os.path.splitext(fnroot)
-        if string.lower(plotMode[1][1:]) in ['pdf','ps','eps','gif','png','jpg','jpeg','pct','pict','tiff','tif','py','bmp']:
+        if string.lower(plotMode[1][1:]) in self._saveModes:
             fnroot = plotMode[0]
 
-        plotMode, verbose = formats or getattr(self,'formats',['pdf']), (verbose is not None and (verbose,) or (getattr(self,'verbose',verbose),))[0]
+        plotMode = map(str.lower,formats or getattr(self,'formats',['pdf']))
+        verbose = (verbose is not None and (verbose,) or (getattr(self,'verbose',verbose),))[0]
         _saved = logger.warnOnce.enabled, logger.infoOnce.enabled
         logger.warnOnce.enabled = logger.infoOnce.enabled = verbose
         if 'pdf' in plotMode:
             from reportlab.graphics import renderPDF
             filename = fnroot+'.pdf'
             if verbose: print "generating PDF file %s" % filename
-            renderPDF.drawToFile(self, filename, title, showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+            renderPDF.drawToFile(self, filename, title, showBoundary=getattr(self,'showBorder',rl_config.showBoundary),**_extraKW(self,'_renderPDF_',**kw))
             ext = ext +  '/.pdf'
             if sys.platform=='mac':
                 import macfs, macostools
                 macfs.FSSpec(filename).SetCreatorType("CARO", "PDF ")
                 macostools.touched(filename)
 
-        for bmFmt in ['gif','png','tif','jpg','tiff','pct','pict', 'bmp']:
+        for bmFmt in ['gif','png','tif','jpg','tiff','pct','pict', 'bmp','tiffp']:
             if bmFmt in plotMode:
                 from reportlab.graphics import renderPM
                 filename = '%s.%s' % (fnroot,bmFmt)
                 if verbose: print "generating %s file %s" % (bmFmt,filename)
-                renderPM.drawToFile(self, filename,fmt=bmFmt,showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+                renderPM.drawToFile(self, filename,fmt=bmFmt,showBoundary=getattr(self,'showBorder',rl_config.showBoundary),**_extraKW(self,'_renderPM_',**kw))
                 ext = ext + '/.' + bmFmt
 
         if 'eps' in plotMode:
-            from rlextra.graphics import renderPS_SEP
+            try:
+                from rlextra.graphics import renderPS_SEP as renderPS
+            except ImportError:
+                from reportlab.graphics import renderPS
             filename = fnroot+'.eps'
             if verbose: print "generating EPS file %s" % filename
-            renderPS_SEP.drawToFile(self,
+            renderPS.drawToFile(self,
                                 filename,
                                 title = fnroot,
                                 dept = getattr(self,'EPS_info',['Testing'])[0],
                                 company = getattr(self,'EPS_info',['','ReportLab'])[1],
-                                preview = getattr(self,'preview',1),
-                                showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+                                preview = getattr(self,'preview',rl_config.eps_preview),
+                                showBoundary=getattr(self,'showBorder',rl_config.showBoundary),
+                                ttf_embed=getattr(self,'ttf_embed',rl_config.eps_ttf_embed),
+                                **_extraKW(self,'_renderPS_',**kw))
             ext = ext +  '/.eps'
 
+        if 'svg' in plotMode:
+            from reportlab.graphics import renderSVG
+            filename = fnroot+'.svg'
+            if verbose: print "generating EPS file %s" % filename
+            renderSVG.drawToFile(self,
+                                filename,
+                                showBoundary=getattr(self,'showBorder',rl_config.showBoundary),**_extraKW(self,'_renderSVG_',**kw))
+            ext = ext +  '/.svg'
+
         if 'ps' in plotMode:
             from reportlab.graphics import renderPS
             filename = fnroot+'.ps'
             if verbose: print "generating EPS file %s" % filename
-            renderPS.drawToFile(self, filename, showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+            renderPS.drawToFile(self, filename, showBoundary=getattr(self,'showBorder',rl_config.showBoundary),**_extraKW(self,'_renderPS_',**kw))
             ext = ext +  '/.ps'
 
         if 'py' in plotMode:
@@ -699,18 +743,22 @@ class Drawing(Group, Flowable):
 
     def asString(self, format, verbose=None, preview=0):
         """Converts to an 8 bit string in given format."""
-        assert format in ['pdf','ps','eps','gif','png','jpg','jpeg','bmp','ppm','tiff','tif','py','pict','pct'], 'Unknown file format "%s"' % format
+        assert format in ['pdf','ps','eps','gif','png','jpg','jpeg','bmp','ppm','tiff','tif','py','pict','pct','tiffp'], 'Unknown file format "%s"' % format
         from reportlab import rl_config
         #verbose = verbose is not None and (verbose,) or (getattr(self,'verbose',verbose),)[0]
         if format == 'pdf':
             from reportlab.graphics import renderPDF
             return renderPDF.drawToString(self)
-        elif format in ['gif','png','tif','jpg','pct','pict','bmp','ppm']:
+        elif format in ['gif','png','tif','tiff','jpg','pct','pict','bmp','ppm','tiffp']:
             from reportlab.graphics import renderPM
             return renderPM.drawToString(self, fmt=format)
         elif format == 'eps':
-            from rlextra.graphics import renderPS_SEP
-            return renderPS_SEP.drawToString(self,
+            try:
+                from rlextra.graphics import renderPS_SEP as renderPS
+            except ImportError:
+                from reportlab.graphics import renderPS
+
+            return renderPS.drawToString(self,
                                 preview = preview,
                                 showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
         elif format == 'ps':
@@ -842,7 +890,7 @@ class Path(SolidShape):
         self.isClipPath = isClipPath
 
     def copy(self):
-        new = Path(self.points[:], self.operators[:])
+        new = self.__class__(self.points[:], self.operators[:])
         new.setProperties(self.getProperties())
         return new
 
@@ -867,26 +915,31 @@ class Path(SolidShape):
 EmptyClipPath=Path()    #special path
 
 def getArcPoints(centerx, centery, radius, startangledegrees, endangledegrees, yradius=None, degreedelta=None, reverse=None):
-    degreedelta = degreedelta or 1
     if yradius is None: yradius = radius
     points = []
     from math import sin, cos, pi
     degreestoradians = pi/180.0
-    radiansdelta = degreedelta*degreestoradians
     startangle = startangledegrees*degreestoradians
     endangle = endangledegrees*degreestoradians
     while endangle<startangle:
         endangle = endangle+2*pi
     angle = float(endangle - startangle)
+    a = points.append
     if angle>.001:
+        degreedelta = min(angle,degreedelta or 1.)
+        radiansdelta = degreedelta*degreestoradians
         n = max(int(angle/radiansdelta+0.5),1)
         radiansdelta = angle/n
-        a = points.append
-        for angle in xrange(n+1):
-            angle = startangle+angle*radiansdelta
-            a((centerx+radius*cos(angle),centery+yradius*sin(angle)))
-        #a((centerx+radius*cos(endangle),centery+yradius*sin(endangle)))
-        if reverse: points.reverse()
+        n += 1
+    else:
+        n = 1
+        radiansdelta = 0
+
+    for angle in xrange(n):
+        angle = startangle+angle*radiansdelta
+        a((centerx+radius*cos(angle),centery+yradius*sin(angle)))
+
+    if reverse: points.reverse()
     return points
 
 class ArcPath(Path):
@@ -942,7 +995,7 @@ class Rect(SolidShape):
         self.ry = ry
 
     def copy(self):
-        new = Rect(self.x, self.y, self.width, self.height)
+        new = self.__class__(self.x, self.y, self.width, self.height)
         new.setProperties(self.getProperties())
         return new
 
@@ -970,7 +1023,7 @@ class Image(SolidShape):
         self.path = path
 
     def copy(self):
-        new = Image(self.x, self.y, self.width, self.height, self.path)
+        new = self.__class__(self.x, self.y, self.width, self.height, self.path)
         new.setProperties(self.getProperties())
         return new
 
@@ -992,7 +1045,7 @@ class Circle(SolidShape):
         self.r = r
 
     def copy(self):
-        new = Circle(self.cx, self.cy, self.r)
+        new = self.__class__(self.cx, self.cy, self.r)
         new.setProperties(self.getProperties())
         return new
 
@@ -1000,7 +1053,6 @@ class Circle(SolidShape):
         return (self.cx - self.r, self.cy - self.r, self.cx + self.r, self.cy + self.r)
 
 class Ellipse(SolidShape):
-
     _attrMap = AttrMap(BASE=SolidShape,
         cx = AttrMapValue(isNumber),
         cy = AttrMapValue(isNumber),
@@ -1016,7 +1068,7 @@ class Ellipse(SolidShape):
         self.ry = ry
 
     def copy(self):
-        new = Ellipse(self.cx, self.cy, self.rx, self.ry)
+        new = self.__class__(self.cx, self.cy, self.rx, self.ry)
         new.setProperties(self.getProperties())
         return new
 
@@ -1069,40 +1121,43 @@ class Wedge(SolidShape):
         yradius, radius1, yradius1 = self._xtraRadii()
         startangledegrees = self.startangledegrees
         endangledegrees = self.endangledegrees
-        degreedelta = self.degreedelta
         from math import sin, cos, pi
         degreestoradians = pi/180.0
-        radiansdelta = degreedelta*degreestoradians
         startangle = startangledegrees*degreestoradians
         endangle = endangledegrees*degreestoradians
         while endangle<startangle:
             endangle = endangle+2*pi
         angle = float(endangle-startangle)
         points = []
-        if angle>.001:
+        if angle>0.001:
+            degreedelta = min(self.degreedelta or 1.,angle)
+            radiansdelta = degreedelta*degreestoradians
             n = max(1,int(angle/radiansdelta+0.5))
             radiansdelta = angle/n
-            a = points.append
-            CA = []
-            CAA = CA.append
-            for angle in xrange(n+1):
-                angle = startangle+angle*radiansdelta
-                CAA((cos(angle),sin(angle)))
-            #CAA((cos(endangle),sin(endangle)))
+            n += 1
+        else:
+            n = 1
+            radiansdelta = 0
+        CA = []
+        CAA = CA.append
+        a = points.append
+        for angle in xrange(n):
+            angle = startangle+angle*radiansdelta
+            CAA((cos(angle),sin(angle)))
+        for c,s in CA:
+            a(centerx+radius*c)
+            a(centery+yradius*s)
+        if (radius1==0 or radius1 is None) and (yradius1==0 or yradius1 is None):
+            a(centerx); a(centery)
+        else:
+            CA.reverse()
             for c,s in CA:
-                a(centerx+radius*c)
-                a(centery+yradius*s)
-            if (radius1==0 or radius1 is None) and (yradius1==0 or yradius1 is None):
-                a(centerx); a(centery)
-            else:
-                CA.reverse()
-                for c,s in CA:
-                    a(centerx+radius1*c)
-                    a(centery+yradius1*s)
+                a(centerx+radius1*c)
+                a(centery+yradius1*s)
         return Polygon(points)
 
     def copy(self):
-        new = Wedge(self.centerx,
+        new = self.__class__(self.centerx,
                     self.centery,
                     self.radius,
                     self.startangledegrees,
@@ -1127,7 +1182,7 @@ class Polygon(SolidShape):
         self.points = points
 
     def copy(self):
-        new = Polygon(self.points)
+        new = self.__class__(self.points)
         new.setProperties(self.getProperties())
         return new
 
@@ -1158,7 +1213,7 @@ class PolyLine(LineShape):
         self.points = points
 
     def copy(self):
-        new = PolyLine(self.points)
+        new = self.__class__(self.points)
         new.setProperties(self.getProperties())
         return new
 
@@ -1178,7 +1233,9 @@ class String(Shape):
         fontSize = AttrMapValue(isNumber),
         fillColor = AttrMapValue(isColorOrNone),
         textAnchor = AttrMapValue(isTextAnchor),
+        encoding = AttrMapValue(isString),
         )
+    encoding = 'utf8'
 
     def __init__(self, x, y, text, **kw):
         self.x = x
@@ -1191,16 +1248,16 @@ class String(Shape):
         self.setProperties(kw)
 
     def getEast(self):
-        return self.x + stringWidth(self.text,self.fontName,self.fontSize)
+        return self.x + stringWidth(self.text,self.fontName,self.fontSize, self.encoding)
 
     def copy(self):
-        new = String(self.x, self.y, self.text)
+        new = self.__class__(self.x, self.y, self.text)
         new.setProperties(self.getProperties())
         return new
 
     def getBounds(self):
         # assumes constant drop of 0.2*size to baseline
-        w = stringWidth(self.text,self.fontName,self.fontSize)
+        w = stringWidth(self.text,self.fontName,self.fontSize,self.encoding)
         if self.textAnchor == 'start':
             x = self.x
         elif self.textAnchor == 'middle':
