@@ -1,8 +1,7 @@
-#copyright ReportLab Inc. 2000
+#Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/lib/abag.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/lib/abag.py,v 1.8 2002/07/24 19:56:37 andy_robinson Exp $
-__version__=''' $Id: abag.py,v 1.8 2002/07/24 19:56:37 andy_robinson Exp $ '''
+#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/lib/abag.py
+__version__=''' $Id: abag.py 3004 2006-12-19 14:50:51Z rgbecker $ '''
 
 class ABag:
     """
@@ -14,12 +13,11 @@ class ABag:
     c = a.clone(ak0=av0,.....) copy with optional additional attributes.
     """
     def __init__(self,**attr):
-        for k,v in attr.items():
-            setattr(self,k,v)
+        self.__dict__.update(attr)
 
     def clone(self,**attr):
-        n = apply(ABag,(),self.__dict__)
-        if attr != {}: apply(ABag.__init__,(n,),attr)
+        n = ABag(**self.__dict__)
+        if attr: n.__dict__.update(attr)
         return n
 
     def __repr__(self):
@@ -42,4 +40,4 @@ if __name__=="__main__":
     AB = ABag(a=1, c="hello")
     CD = AB.clone()
     print AB
-    print CD
\ No newline at end of file
+    print CD
