@@ -1,8 +1,7 @@
-#copyright ReportLab Inc. 2000
+#Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/pdfbase/pdfdoc.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/pdfbase/pdfdoc.py,v 1.90 2004/01/20 22:50:31 andy_robinson Exp $
-__version__=''' $Id: pdfdoc.py,v 1.90 2004/01/20 22:50:31 andy_robinson Exp $ '''
+#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfbase/pdfdoc.py
+__version__=''' $Id: pdfdoc.py 3083 2007-05-14 16:05:04Z rgbecker $ '''
 __doc__="""
 The module pdfdoc.py handles the 'outer structure' of PDF documents, ensuring that
 all objects are properly cross-referenced and indexed to the nearest byte.  The
@@ -15,12 +14,12 @@ The classes within this generally mirror structures in the PDF file
 and are not part of any public interface.  Instead, canvas and font
 classes are made available elsewhere for users to manipulate.
 """
-
-import string, types
+import string, types, binascii, codecs
 from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase.pdfutils import LINEEND # this constant needed in both
 from reportlab import rl_config
 from reportlab.lib.utils import import_zlib, open_for_read, fp_str
+from reportlab.pdfbase import pdfmetrics
 
 from sys import platform
 try:
@@ -38,10 +37,15 @@ if platform[:4] == 'java' and version_info[:2] == (2, 1):
             return x
         return map(f, sequence)
 
+def utf8str(x):
+    if isinstance(x,unicode):
+        return x.encode('utf8')
+    else:
+        return str(x)
+
 class PDFError(Exception):
     pass
 
-
 # set this flag to get more vertical whitespace (and larger files)
 LongFormat = 1
 ##if LongFormat: (doesn't work)
@@ -72,48 +76,39 @@ Pages = "Pages"
 
 ### generic utilities
 
-
 # for % substitutions
 LINEENDDICT = {"LINEEND": LINEEND, "PERCENT": "%"}
 
-def format(element, document, toplevel=0):
+from types import InstanceType
+def format(element, document, toplevel=0, InstanceType=InstanceType):
     """Indirection step for formatting.
        Ensures that document parameters alter behaviour
        of formatting for all elements.
     """
-    from types import InstanceType, FloatType, IntType
-    if type(element) is InstanceType:
+    t=type(element)
+    if t is InstanceType:
         if not toplevel and hasattr(element, __RefOnly__):
             # the object cannot be a component at non top level.
             # make a reference to it and return it's format
-            R = document.Reference(element)
-            return R.format(document)
+            return document.Reference(element).format(document)
         else:
-            try:
-                fmt = element.format
-            except:
-                raise AttributeError, "%s has no format operation" % element
-            f = fmt(document)
-            if DoComments and hasattr(element, __Comment__):
+            f = element.format(document)
+            if not rl_config.invariant and DoComments and hasattr(element, __Comment__):
                 f = "%s%s%s%s" % ("% ", element.__Comment__, LINEEND, f)
             return f
-    elif type(element) in (FloatType, IntType):
+    elif t in (float, int):
         #use a controlled number formatting routine
         #instead of str, so Jython/Python etc do not differ
         return fp_str(element)
     else:
         return str(element)
 
-def indent(s, IND=LINEEND+" "):
-    return string.replace(s, LINEEND, IND)
-
 def xObjectName(externalname):
     return "FormXob.%s" % externalname
 
 # backwards compatibility
 formName = xObjectName
 
-
 # no encryption
 class NoEncryption:
     def encode(self, t):
@@ -134,7 +129,6 @@ class DummyDoc:
     encrypt = NoEncryption()
 
 ### the global document structure manager
-
 class PDFDocument:
     _ID = None
     objectcounter = 0
@@ -144,18 +138,10 @@ class PDFDocument:
     encrypt = NoEncryption() # default no encryption
     pageCounter = 1
     def __init__(self,
-                 encoding=rl_config.defaultEncoding,
                  dummyoutline=0,
                  compression=rl_config.pageCompression,
-                 invariant=rl_config.invariant):
-        #self.defaultStreamFilters = [PDFBase85Encode, PDFZCompress] # for testing!
-        #self.defaultStreamFilters = [PDFZCompress] # for testing!
-        assert encoding in ['MacRomanEncoding',
-                            'WinAnsiEncoding',
-                            'MacRoman',
-                            'WinAnsi'], 'Unsupported encoding %s' % encoding
-        if encoding[-8:] <> 'Encoding':
-            encoding = encoding + 'Encoding'
+                 invariant=rl_config.invariant,
+                 filename=None):
 
         # allow None value to be passed in to mean 'give system defaults'
         if invariant is None:
@@ -163,7 +149,6 @@ class PDFDocument:
         else:
             self.invariant = invariant
         self.setCompression(compression)
-        self.encoding = encoding
         # signature for creating PDF ID
         import md5
         sig = self.signature = md5.new()
@@ -208,7 +193,7 @@ class PDFDocument:
     def updateSignature(self, thing):
         "add information to the signature"
         if self._ID: return # but not if its used already!
-        self.signature.update(str(thing))
+        self.signature.update(utf8str(thing))
 
     def ID(self):
         "A unique fingerprint for the file (unless in invariant mode)"
@@ -216,7 +201,7 @@ class PDFDocument:
             return self._ID
         digest = self.signature.digest()
         doc = DummyDoc()
-        ID = PDFString(digest)
+        ID = PDFString(digest,enc='raw')
         IDs = ID.format(doc)
         self._ID = "%s %% ReportLab generated PDF document -- digest (http://www.reportlab.com) %s [%s %s] %s" % (
                 LINEEND, LINEEND, IDs, IDs, LINEEND)
@@ -226,10 +211,10 @@ class PDFDocument:
         if callable(getattr(filename, "write",None)):
             myfile = 0
             f = filename
-            filename = str(getattr(filename,'name',''))
+            filename = utf8str(getattr(filename,'name',''))
         else :
             myfile = 1
-            filename = str(filename)
+            filename = utf8str(filename)
             f = open(filename, "wb")
         f.write(self.GetPDFData(canvas))
         if myfile:
@@ -277,9 +262,8 @@ class PDFDocument:
         else:
             try:
                 # does pdfmetrics know about it? if so, add
-                from reportlab.pdfbase import pdfmetrics
                 fontObj = pdfmetrics.getFont(psfontname)
-                if getattr(fontObj, '_dynamicFont', 0):
+                if fontObj._dynamicFont:
                     raise PDFError, "getInternalFontName(%s) called for a dynamic font" % repr(psfontname)
                 fontObj.addObjects(self)
                 #self.addFont(fontObj)
@@ -300,7 +284,6 @@ class PDFDocument:
         self.pageCounter = self.pageCounter+1
         self.inObject = None
 
-
     def addForm(self, name, form):
         """add a Form XObject."""
         # XXX should check that name is a legal PDF name
@@ -321,15 +304,39 @@ class PDFDocument:
 
     def setTitle(self, title):
         "embeds in PDF file"
-        self.info.title = title
+        if title is None:
+            self.info.title = '(anonymous)'
+        else:
+            self.info.title = title
 
     def setAuthor(self, author):
         "embedded in PDF file"
-        self.info.author = author
+        #allow resetting to clear it
+        if author is None:
+            self.info.author = '(anonymous)'
+        else:
+            self.info.author = author
 
     def setSubject(self, subject):
         "embeds in PDF file"
-        self.info.subject = subject
+
+        #allow resetting to clear it
+        if subject is None:
+            self.info.subject = '(unspecified)'
+        else:
+            self.info.subject = subject
+
+    def setKeywords(self, keywords):
+        "embeds a string containing keywords in PDF file"
+
+        #allow resetting to clear it but ensure it's a string
+        if keywords is None:
+            self.info.keywords = ''
+        else:
+            self.info.keywords = keywords
+
+    def setDateFormatter(self, dateFormatter):
+        self.info._dateFormatter = dateFormatter
 
     def getAvailableFonts(self):
         fontnames = self.fontMapping.keys()
@@ -341,8 +348,6 @@ class PDFDocument:
         fontnames.sort()
         return fontnames
 
-
-
     def format(self):
         # register the Catalog/INfo and then format the objects one by one until exhausted
         # (possible infinite loop if there is a bug that continually makes new objects/refs...)
@@ -368,7 +373,7 @@ class PDFDocument:
         done = None
         File = PDFFile() # output collector
         while done is None:
-            counter = counter+1 # do next object...
+            counter += 1 # do next object...
             if numbertoid.has_key(counter):
                 id = numbertoid[counter]
                 #printidToOb
@@ -378,7 +383,7 @@ class PDFDocument:
                 #encrypt.register(id,
                 IOf = IO.format(self)
                 # add a comment to the PDF output
-                if DoComments:
+                if not rl_config.invariant and DoComments:
                     try:
                         classname = obj.__class__.__name__
                     except:
@@ -417,17 +422,21 @@ class PDFDocument:
         internalname = xObjectName(name)
         return self.idToObject.has_key(internalname)
 
-    def getFormBBox(self, name):
-        "get the declared bounding box of the form as a list"
+    def getFormBBox(self, name, boxType="MediaBox"):
+        """get the declared bounding box of the form as a list.
+        If you specify a different PDF box definition (e.g. the
+        ArtBox) and it has one, that's what you'll get."""
         internalname = xObjectName(name)
         if self.idToObject.has_key(internalname):
             theform = self.idToObject[internalname]
+            if hasattr(theform,'_extra_pageCatcher_info'):
+                return theform._extra_pageCatcher_info[boxType]
             if isinstance(theform, PDFFormXObject):
                 # internally defined form
                 return theform.BBoxList()
             elif isinstance(theform, PDFStream):
                 # externally defined form
-                return list(theform.dictionary.dict["BBox"].sequence)
+                return list(theform.dictionary.dict[boxType].sequence)
             else:
                 raise ValueError, "I don't understand the form instance %s" % repr(name)
 
@@ -447,10 +456,9 @@ class PDFDocument:
         #print "xobjDict D", D
         return PDFDictionary(D)
 
-    def Reference(self, object, name=None):
+    def Reference(self, object, name=None, InstanceType=InstanceType):
         ### note references may "grow" during the final formatting pass: don't use d.keys()!
         # don't make references to other references, or non instances, unless they are named!
-        from types import InstanceType
         #print"object type is ", type(object)
         tob = type(object)
         idToObject = self.idToObject
@@ -484,7 +492,6 @@ class PDFDocument:
         return PDFObjectReference(name)
 
 ### chapter 4 Objects
-
 PDFtrue = "true"
 PDFfalse = "false"
 PDFnull = "null"
@@ -493,18 +500,7 @@ class PDFText:
     def __init__(self, t):
         self.t = t
     def format(self, document):
-        t = self.t
-        t = document.encrypt.encode(t)
-        L = list(t)
-        for i in range(len(L)):
-            ch = L[i]
-            n = ord(ch)
-            h = hex(n)
-            h2 = h[2:] # nuke the 0x
-            if len(h2)<2:
-                h2 = "0"+h2
-            L[i] = h2
-        result = string.join(L, "")
+        result = binascii.hexlify(document.encrypt.encode(self.t))
         return "<%s>" % result
     def __str__(self):
         dummydoc = DummyDoc()
@@ -513,37 +509,100 @@ class PDFText:
 def PDFnumber(n):
     return n
 
+import re
+_re_cleanparens=re.compile('[^()]')
+del re
+def _isbalanced(s):
+    '''test whether a string is balanced in parens'''
+    s = _re_cleanparens.sub('',s)
+    n = 0
+    for c in s:
+        if c=='(': n+=1
+        else:
+            n -= 1
+            if n<0: return 0
+    return not n and 1 or 0
+
+def _checkPdfdoc(utext):
+    '''return true if no Pdfdoc encoding errors'''
+    try:
+        utext.encode('pdfdoc')
+        return 1
+    except UnicodeEncodeError, e:
+        return 0
+
 class PDFString:
-    def __init__(self, str):
-        # might need to change this to class for encryption
-        self.s = str
+    def __init__(self, s, escape=1, enc='auto'):
+        '''s can be unicode/utf8 or a PDFString
+        if escape is true then the output will be passed through escape
+        if enc is raw then the string will be left alone
+        if enc is auto we'll try and automatically adapt to utf_16_be if the
+        effective string is not entirely in pdfdoc
+        '''
+        if isinstance(s,PDFString):
+            self.s = s.s
+            self.escape = s.escape
+            self.enc = s.enc
+        else:
+            self.s = s
+            self.escape = escape
+            self.enc = enc
     def format(self, document):
-        s = document.encrypt.encode(self.s)
-        try:
-            return "(%s)" % pdfutils._escape(s)
-        except:
-            raise ValueError, "cannot escape %s %s" %(s, repr(s))
+        s = self.s
+        enc = getattr(self,'enc','auto')
+        if type(s) is str:
+            if enc is 'auto':
+                try:
+                    u = s.decode(s.startswith(codecs.BOM_UTF16_BE) and 'utf16' or 'utf8')
+                except:
+                    import sys
+                    print >>sys.stderr, 'Error in',repr(s)
+                    raise
+                if _checkPdfdoc(u):
+                    s = u.encode('pdfdoc')
+                else:
+                    s = codecs.BOM_UTF16_BE+u.encode('utf_16_be')
+        elif type(s) is unicode:
+            if enc is 'auto':
+                if _checkPdfdoc(s):
+                    s = s.encode('pdfdoc')
+                else:
+                    s = codecs.BOM_UTF16_BE+s.encode('utf_16_be')
+            else:
+                s = codecs.BOM_UTF16_BE+s.encode('utf_16_be')
+        else:
+            raise ValueError('PDFString argument must be str/unicode not %s' % type(s))
+
+        escape = getattr(self,'escape',1)
+        if not isinstance(document.encrypt,NoEncryption):
+            s = document.encrypt.encode(s)
+            escape = 1
+        if escape:
+            try:
+                es = "(%s)" % pdfutils._escape(s)
+            except:
+                raise ValueError("cannot escape %s %s" % (s, repr(s)))
+            if escape&2:
+                es = es.replace('\\012','\n')
+            if escape&4 and _isbalanced(s): 
+                es = es.replace('\\(','(').replace('\\)',')')
+            return es
+        else:
+            return '(%s)' % s
     def __str__(self):
         return "(%s)" % pdfutils._escape(self.s)
 
-def PDFName(data):
+def PDFName(data,lo=chr(0x21),hi=chr(0x7e)):
     # might need to change this to class for encryption
     #  NOTE: RESULT MUST ALWAYS SUPPORT MEANINGFUL COMPARISONS (EQUALITY) AND HASH
     # first convert the name
-    ldata = list(data)
-    index = 0
-    for thischar in data:
-        if 0x21<=ord(thischar)<=0x7e and thischar not in "%()<>{}[]#":
-            pass # no problemo
-        else:
-            hexord = hex(ord(thischar))[2:] # forget the 0x thing...
-            ldata[index] = "#"+hexord
-        index = index+1
-    data = string.join(ldata, "")
-    return "/%s" % data
+    L = list(data)
+    for i,c in enumerate(L):
+        if c<lo or c>hi or c in "%()<>{}[]#":
+            L[i] = "#"+hex(ord(c))[2:] # forget the 0x thing...
+    return "/"+(''.join(L))
 
 class PDFDictionary:
-
     multiline = LongFormat
     def __init__(self, dict=None):
         """dict should be namestring to value eg "a": 122 NOT pdfname to value NOT "/a":122"""
@@ -553,34 +612,29 @@ class PDFDictionary:
             self.dict = dict.copy()
     def __setitem__(self, name, value):
         self.dict[name] = value
-    def Reference(name, document):
-        ob = self.dict[name]
-        self.dict[name] = document.Reference(ob)
-    def format(self, document):
+    def Reference(self, name, document):
+        self.dict[name] = document.Reference(self.dict[name])
+    def format(self, document,IND=LINEEND+' '):
         dict = self.dict
         keys = dict.keys()
         keys.sort()
-        L = []
-        a = L.append
-        for k in keys:
-            v = dict[k]
-            fv = format(v, document)
-            fk = format(PDFName(k), document)
-            #a(fk)
-            #a(" "+fv)
-            a(fk + " " + fv)
-        #L = map(str, L)
+        L = [(format(PDFName(k),document)+" "+format(dict[k],document)) for k in keys]
         if self.multiline:
-            Lj = string.join(L, LINEEND)
-            Lj = indent(Lj)
+            L = IND.join(L)
         else:
-            Lj = L
             # break up every 6 elements anyway
-            for i in range(6, len(Lj), 6):
-                Lj.insert(i,LINEEND)
-            Lj = string.join(L, " ")
-        return "<< %s >>" % Lj
+            t=L.insert
+            for i in xrange(6, len(L), 6):
+                t(i,LINEEND)
+            L = " ".join(L)
+        return "<< %s >>" % L
 
+    def copy(self):
+        return PDFDictionary(self.dict)
+
+    def has_key(self,k):
+        return self.dict.has_key(k)
+    
 # stream filters are objects to support round trip and
 # possibly in the future also support parameters
 class PDFStreamFilterZCompress:
@@ -603,7 +657,10 @@ class PDFStreamFilterBase85Encode:
     pdfname = "ASCII85Decode"
     def encode(self, text):
         from pdfutils import _AsciiBase85Encode, _wrap
-        return _wrap(_AsciiBase85Encode(text))
+        text = _AsciiBase85Encode(text)
+        if rl_config.wrapA85:
+            text = _wrap(text)
+        return text
     def decode(self, text):
         from pdfutils import _AsciiBase85Decode
         return _AsciiBase85Decode(text)
@@ -691,24 +748,17 @@ class PDFArray:
     def References(self, document):
         """make all objects in sequence references"""
         self.sequence = map(document.Reference, self.sequence)
-    def format(self, document):
-        #ssequence = map(str, self.sequence)
-        sequence = self.sequence
-        fsequence = []
-        for elt in sequence:
-            felt = format(elt, document)
-            fsequence.append(felt)
+    def format(self, document, IND=LINEEND+' '):
+        L = [format(e, document) for e in self.sequence]
         if self.multiline:
-            Lj = string.join(fsequence, LINEEND)
-            Lj = indent(Lj)
+            L = IND.join(L)
         else:
             # break up every 10 elements anyway
-            Lj = fsequence
             breakline = LINEEND+" "
-            for i in range(10, len(Lj), 10):
-                Lj.insert(i,breakline)
-            Lj = string.join(Lj)
-        return "[ %s ]" % Lj
+            for i in xrange(10, len(L), 10):
+                L.insert(i,breakline)
+            L = ' '.join(L)
+        return "[ %s ]" % L
 
 INDIRECTOBFMT = ("%(n)s %(v)s obj%(LINEEND)s"
                  "%(content)s" "%(LINEEND)s"
@@ -736,12 +786,10 @@ class PDFObjectReference:
     def __init__(self, name):
         self.name = name
     def format(self, document):
-        name = self.name
         try:
-            (n, v) = document.idToObjectNumberAndVersion[name]
+            return "%s %s R" % document.idToObjectNumberAndVersion[self.name]
         except:
-            raise KeyError, "forward reference to %s not resolved upon final formatting" % repr(name)
-        return "%s %s R" % (n,v)
+            raise KeyError, "forward reference to %s not resolved upon final formatting" % repr(self.name)
 
 ### chapter 5
 # Following Ken Lunde's advice and the PDF spec, this includes
@@ -757,17 +805,22 @@ class PDFFile:
     ### just accumulates strings: keeps track of current offset
     def __init__(self):
         self.strings = []
+        self.write = self.strings.append
         self.offset = 0
         self.add(PDFHeader)
+
+    def closeOrReset(self):
+        pass
+
     def add(self, s):
         """should be constructed as late as possible, return position where placed"""
         result = self.offset
         self.offset = result+len(s)
-        self.strings.append(s)
+        self.write(s)
         return result
     def format(self, document):
         strings = map(str, self.strings) # final conversion, in case of lazy objects
-        return string.join(self.strings, "")
+        return string.join(strings, "")
 
 XREFFMT = '%0.10d %0.5d n'
 
@@ -914,7 +967,7 @@ class PDFCatalog:
 #  ViewerPreferences, PageLabelDictionaries,
 
 class PDFPages(PDFCatalog):
-    """PAGES TREE WITH ONE INTERNAL NODE, FOR "BALANCING" CHANGE IMPLEMENATION"""
+    """PAGES TREE WITH ONE INTERNAL NODE, FOR "BALANCING" CHANGE IMPLEMENTATION"""
     __Comment__ = "page tree"
     __RefOnly__ = 1
     # note: could implement page attribute inheritance...
@@ -931,12 +984,12 @@ class PDFPages(PDFCatalog):
     def check_format(self, document):
         # convert all pages to page references
         pages = self.pages
-        kids = PDFArray(self.pages)
+        kids = PDFArray(pages)
         # make sure all pages are references
         kids.References(document)
         self.Kids = kids
         self.Count = len(pages)
-
+ 
 class PDFPage(PDFCatalog):
     __Comment__ = "Page dictionary"
     # all PDF attributes can be set explicitly
@@ -985,7 +1038,7 @@ class PDFPage(PDFCatalog):
         # set up parameters unless usual behaviour is suppressed
         if self.Override_default_compilation:
             return
-        self.MediaBox = self.MediaBox or PDFArray([0, 0, self.pagewidth, self.pageheight])
+        self.MediaBox = self.MediaBox or PDFArray(self.Rotate in (90,270) and [0,0,self.pageheight,self.pagewidth] or [0, 0, self.pagewidth, self.pageheight])
         if not self.Annots:
             self.Annots = None
         else:
@@ -1033,7 +1086,6 @@ def testpage(document):
     pages.addPage(P)
 
 #### DUMMY OUTLINES IMPLEMENTATION FOR testing
-
 DUMMYOUTLINE = """
 <<
   /Count
@@ -1049,7 +1101,6 @@ class PDFOutlines0:
     def format(self, document):
         return self.text
 
-
 class OutlineEntryObject:
     "an entry in an outline"
     Title = Dest = Parent = Prev = Next = First = Last = Count = None
@@ -1065,7 +1116,6 @@ class OutlineEntryObject:
         PD = PDFDictionary(D)
         return PD.format(document)
 
-
 class PDFOutlines:
     """takes a recursive list of outline destinations
        like
@@ -1253,7 +1303,7 @@ class PDFOutlines:
             try:
                 [(Title, Dest)] = leafdict.items()
             except:
-                raise ValueError, "bad outline leaf dictionary, should have one entry "+str(elt)
+                raise ValueError, "bad outline leaf dictionary, should have one entry "+utf8str(elt)
             eltobj.Title = destinationnamestotitles[Title]
             eltobj.Dest = Dest
             if te is TupleType and closedict.has_key(Dest):
@@ -1280,66 +1330,64 @@ def count(tree, closedict=None):
         return reduce(add, counts)
     return 1
 
-
-
-#### dummy info
-DUMMYINFO = """
-<</Title (testing)
-/Author (arw)
-/CreationDate (D:20001012220652)
-/Producer (ReportLab http://www.reportlab.com)
-/Subject (this file generated by an alpha test module)
->>
-"""
-class PDFInfo0:
-    __Comment__ = "TEST INFO STRUCTURE"
-    text = string.replace(DUMMYINFO, "\n", LINEEND)
-    __RefOnly__ = 1
-    def format(self, document):
-        return self.text
-
 class PDFInfo:
     """PDF documents can have basic information embedded, viewable from
     File | Document Info in Acrobat Reader.  If this is wrong, you get
     Postscript errors while printing, even though it does not print."""
+    producer = "ReportLab http://www.reportlab.com"
+    title = "untitled"
+    author = "anonymous"
+    subject = "unspecified"
+    keywords = ""
+    _dateFormatter = None
+
     def __init__(self):
         self.invariant = rl_config.invariant
-        self.title = "untitled"
-        self.author = "anonymous"
-        self.subject = "unspecified"
 
     def digest(self, md5object):
         # add self information to signature
-        for x in (self.title, self.author, self.subject):
-            md5object.update(str(x))
+        for x in (self.title, self.author, self.subject, self.keywords):
+            md5object.update(utf8str(x))
 
     def format(self, document):
         D = {}
         D["Title"] = PDFString(self.title)
         D["Author"] = PDFString(self.author)
-        D["CreationDate"] = PDFDate(invariant=self.invariant)
-        D["Producer"] = PDFString("ReportLab http://www.reportlab.com")
+        D["CreationDate"] = PDFDate(invariant=self.invariant,dateFormatter=self._dateFormatter)
+        D["Producer"] = PDFString(self.producer)
         D["Subject"] = PDFString(self.subject)
+        D["Keywords"] = PDFString(self.keywords)
+            
         PD = PDFDictionary(D)
         return PD.format(document)
 
+    def copy(self):
+        "shallow copy - useful in pagecatchering"
+        thing = self.__klass__()
+        for (k, v) in self.__dict__.items():
+            setattr(thing, k, v)
+        return thing
 # skipping thumbnails, etc
 
-
 class Annotation:
     """superclass for all annotations."""
     defaults = [("Type", PDFName("Annot"),)]
     required = ("Type", "Rect", "Contents", "Subtype")
     permitted = required+(
-      "Border", "C", "T", "M", "F", "H", "BS", "AA", "AS", "Popup", "P")
-    def cvtdict(self, d):
+      "Border", "C", "T", "M", "F", "H", "BS", "AA", "AS", "Popup", "P", "AP")
+    def cvtdict(self, d, escape=1):
         """transform dict args from python form to pdf string rep as needed"""
         Rect = d["Rect"]
         if type(Rect) is not types.StringType:
             d["Rect"] = PDFArray(Rect)
-        d["Contents"] = PDFString(d["Contents"])
+        d["Contents"] = PDFString(d["Contents"],escape)
         return d
     def AnnotationDict(self, **kw):
+        if 'escape' in kw:
+            escape = kw['escape']
+            del kw['escape']
+        else:
+            escape = 1
         d = {}
         for (name,val) in self.defaults:
             d[name] = val
@@ -1347,7 +1395,7 @@ class Annotation:
         for name in self.required:
             if not d.has_key(name):
                 raise ValueError, "keyword argument %s missing" % name
-        d = self.cvtdict(d)
+        d = self.cvtdict(d,escape=escape)
         permitted = self.permitted
         for name in d.keys():
             if name not in permitted:
@@ -1363,7 +1411,7 @@ class Annotation:
 
 class TextAnnotation(Annotation):
     permitted = Annotation.permitted + (
-        "Open", "Name", "AP")
+        "Open", "Name")
     def __init__(self, Rect, Contents, **kw):
         self.Rect = Rect
         self.Contents = Contents
@@ -1374,7 +1422,23 @@ class TextAnnotation(Annotation):
         d["Rect"] = self.Rect
         d["Contents"] = self.Contents
         d["Subtype"] = "/Text"
-        return apply(self.AnnotationDict, (), d)
+        return self.AnnotationDict(**d)
+
+class FreeTextAnnotation(Annotation):
+    permitted = Annotation.permitted + ("DA",)
+    def __init__(self, Rect, Contents, DA, **kw):
+        self.Rect = Rect
+        self.Contents = Contents
+        self.DA = DA
+        self.otherkw = kw
+    def Dict(self):
+        d = {}
+        d.update(self.otherkw)
+        d["Rect"] = self.Rect
+        d["Contents"] = self.Contents
+        d["DA"] = self.DA
+        d["Subtype"] = "/FreeText"
+        return self.AnnotationDict(**d)
 
 class LinkAnnotation(Annotation):
 
@@ -1403,13 +1467,9 @@ class LinkAnnotation(Annotation):
         d["Dest"] = self.Destination
         return apply(self.AnnotationDict, (), d)
 
-
 # skipping names tree
-
 # skipping actions
-
 # skipping names trees
-
 # skipping to chapter 7
 
 class PDFRectangle:
@@ -1429,25 +1489,27 @@ def _getTimeStamp():
 
 class PDFDate:
     # gmt offset not yet suppported
-    def __init__(self, yyyy=None, mm=None, dd=None, hh=None, m=None, s=None, invariant=rl_config.invariant):
+    def __init__(self, yyyy=None, mm=None, dd=None, hh=None, m=None, s=None,
+                    invariant=rl_config.invariant,dateFormatter=None):
         if None in (yyyy, mm, dd, hh, m, s):
             if invariant:
                 now = (2000,01,01,00,00,00,0)
             else:
                 import time
                 now = tuple(time.localtime(_getTimeStamp())[:6])
-        if yyyy is None: yyyy=now[0]
-        if mm is None: mm=now[1]
-        if dd is None: dd=now[2]
-        if hh is None: hh=now[3]
-        if m is None: m=now[4]
-        if s is None: s=now[5]
-        self.yyyy=yyyy; self.mm=mm; self.dd=dd; self.hh=hh; self.m=m; self.s=s
+            if yyyy is None: yyyy=now[0]
+            if mm is None: mm=now[1]
+            if dd is None: dd=now[2]
+            if hh is None: hh=now[3]
+            if m is None: m=now[4]
+            if s is None: s=now[5]
+        self.date=(yyyy,mm,dd,hh,m,s)
+        self.dateFormatter = dateFormatter
 
     def format(self, doc):
-        S = PDFString('%04d%02d%02d%02d%02d%02d' % (self.yyyy, self.mm, self.dd, self.hh, self.m, self.s))
-        return format(S, doc)
-
+        dfmt = self.dateFormatter or (
+                lambda yyyy,mm,dd,hh,m,s: '%04d%02d%02d%02d%02d%02d' % (yyyy,mm,dd,hh,m,s))
+        return format(PDFString(dfmt(*self.date)), doc)
 
 class Destination:
     """not a pdfobject!  This is a placeholder that can delegates
@@ -1537,7 +1599,7 @@ class PDFDestinationFitV:
         A = PDFArray( [ pageref, PDFName(self.typename), self.left ] )
         return format(A, document)
 
-class PDFDestinationBV(PDFDestinationFitV):
+class PDFDestinationFitBV(PDFDestinationFitV):
     typename = "FitBV"
 
 class PDFDestinationFitR:
@@ -1597,14 +1659,12 @@ class PDFResourceDictionary:
         DD = PDFDictionary(D)
         return format(DD, document)
 
-    ##############################################################################
-    #
-    #   Font objects - the PDFDocument.addFont() method knows which of these
-    #   to construct when given a user-facing Font object
-    #
-    ##############################################################################
-
-
+##############################################################################
+#
+#   Font objects - the PDFDocument.addFont() method knows which of these
+#   to construct when given a user-facing Font object
+#
+##############################################################################
 class PDFType1Font:
     """no init: set attributes explicitly"""
     __RefOnly__ = 1
@@ -1668,10 +1728,6 @@ class PDFTrueTypeFont(PDFType1Font):
 # UGLY ALERT - this needs turning into something O-O, it was hacked
 # across from the pdfmetrics.Encoding class to avoid circularity
 
-
-
-
-
 # skipping CMaps
 
 class PDFFormXObject:
@@ -1737,7 +1793,6 @@ class PDFFormXObject:
         sdict["Resources"] = resources
         return self.Contents.format(document)
 
-
 class PDFPostScriptXObject:
     "For embedding PD (e.g. tray commands) in PDF"
     def __init__(self, content=None):
@@ -1752,6 +1807,7 @@ class PDFPostScriptXObject:
         sdict["Subtype"] = PDFName("PS")
         return S.format(document)
 
+_mode2CS={'RGB':'DeviceRGB', 'L':'DeviceGray', 'CMYK':'DeviceCMYK'}
 class PDFImageXObject:
     # first attempts at a hard-coded one
     # in the file, Image XObjects are stream objects.  We already
@@ -1775,31 +1831,36 @@ class PDFImageXObject:
 
         if source is None:
             pass # use the canned one.
-        elif type(source) == type(''):
+        elif type(source)==type(''):
             # it is a filename
             import os
             ext = string.lower(os.path.splitext(source)[1])
-            if ext in ('.jpg', '.jpeg'):
-                self.loadImageFromJPEG(open_for_read(source))
-            else:
-                self.loadImageFromA85(source)
+            src = open_for_read(source)
+            if not(ext in ('.jpg', '.jpeg') and self.loadImageFromJPEG(src)):
+                self.loadImageFromA85(src)
         else: # it is already a PIL Image
             self.loadImageFromSRC(source)
 
     def loadImageFromA85(self,source):
         IMG=[]
-        imagedata = map(string.strip,pdfutils.cacheImageFile(source,returnInMemory=1,IMG=IMG))
+        imagedata = map(string.strip,pdfutils.makeA85Image(source,IMG=IMG))
         words = string.split(imagedata[1])
         self.width, self.height = map(string.atoi,(words[1],words[3]))
-        self.colorSpace = 'DeviceRGB'
+        self.colorSpace = {'/RGB':'DeviceRGB', '/G':'DeviceGray', '/CMYK':'DeviceCMYK'}[words[7]]
         self.bitsPerComponent = 8
         self._filters = 'ASCII85Decode','FlateDecode' #'A85','Fl'
         if IMG: self._checkTransparency(IMG[0])
-        else: self.mask = None
+        elif self.mask=='auto': self.mask = None
         self.streamContent = string.join(imagedata[3:-1],'')
 
     def loadImageFromJPEG(self,imageFile):
-        info = pdfutils.readJPEGInfo(imageFile)
+        try:
+            try:
+                info = pdfutils.readJPEGInfo(imageFile)
+            finally:
+                imageFile.seek(0) #reset file pointer
+        except:
+            return False
         self.width, self.height = info[0], info[1]
         self.bitsPerComponent = 8
         if info[2] == 1:
@@ -1808,10 +1869,11 @@ class PDFImageXObject:
             self.colorSpace = 'DeviceRGB'
         else: #maybe should generate an error, is this right for CMYK?
             self.colorSpace = 'DeviceCMYK'
-        imageFile.seek(0) #reset file pointer
+            self._dotrans = 1
         self.streamContent = pdfutils._AsciiBase85Encode(imageFile.read())
         self._filters = 'ASCII85Decode','DCTDecode' #'A85','DCT'
         self.mask = None
+        return True
 
     def _checkTransparency(self,im):
         if self.mask=='auto':
@@ -1821,20 +1883,25 @@ class PDFImageXObject:
             else:
                 self.mask = None
         elif hasattr(self.mask,'rgb'):
-            self.mask = self.mask.rgb()+self.mask.rgb()
+            _ = self.mask.rgb()
+            self.mask = _[0],_[0],_[1],_[1],_[2],_[2]
 
     def loadImageFromSRC(self, im):
         "Extracts the stream, width and height"
-        zlib = import_zlib()
-        if not zlib: return
-        self.width, self.height = im.getSize()
-        raw = im.getRGBData()
-        assert(len(raw) == self.width*self.height, "Wrong amount of data for image")
-        self.streamContent = pdfutils._AsciiBase85Encode(zlib.compress(raw))
-        self.colorSpace = 'DeviceRGB'
-        self.bitsPerComponent = 8
-        self._filters = 'ASCII85Decode','FlateDecode' #'A85','Fl'
-        self._checkTransparency(im)
+        fp = im.jpeg_fh()
+        if fp:
+            self.loadImageFromJPEG(fp)
+        else:
+            zlib = import_zlib()
+            if not zlib: return
+            self.width, self.height = im.getSize()
+            raw = im.getRGBData()
+            assert(len(raw) == self.width*self.height, "Wrong amount of data for image")
+            self.streamContent = pdfutils._AsciiBase85Encode(zlib.compress(raw))
+            self.colorSpace= _mode2CS[im.mode]
+            self.bitsPerComponent = 8
+            self._filters = 'ASCII85Decode','FlateDecode' #'A85','Fl'
+            self._checkTransparency(im)
 
     def format(self, document):
         S = PDFStream()
@@ -1846,14 +1913,12 @@ class PDFImageXObject:
         dict["Height"] = self.height
         dict["BitsPerComponent"] = self.bitsPerComponent
         dict["ColorSpace"] = PDFName(self.colorSpace)
+        if self.colorSpace=='DeviceCMYK' and getattr(self,'_dotrans',0):
+            dict["Decode"] = PDFArray([1,0,1,0,1,0,1,0])
         dict["Filter"] = PDFArray(map(PDFName,self._filters))
         dict["Length"] = len(self.streamContent)
-
-        if self.mask:
-            dict["Mask"] = PDFArray(self.mask)
-
+        if self.mask: dict["Mask"] = PDFArray(self.mask)
         return S.format(document)
 
-
 if __name__=="__main__":
     print "There is no script interpretation for pdfdoc."
