@@ -1,20 +1,16 @@
-#copyright ReportLab Inc. 2000
+#Copyright ReportLab Europe Ltd. 2000-2006
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/pdfgen/canvas.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/pdfgen/canvas.py,v 1.120 2004/01/20 22:50:31 andy_robinson Exp $
-__version__=''' $Id: canvas.py,v 1.120 2004/01/20 22:50:31 andy_robinson Exp $ '''
+__version__=''' $Id: canvas.py 3063 2007-04-13 09:52:08Z rgbecker $ '''
 __doc__="""
 The Canvas object is the primary interface for creating PDF files. See
 doc/userguide.pdf for copious examples.
 """
-
-
-
 ENABLE_TRACKING = 1 # turn this off to do profile testing w/o tracking
 
 import os
 import sys
-from string import join, split, strip, atoi, replace, upper
+import re
+from string import join, split, strip, atoi, replace, upper, digits
 import tempfile
 from types import *
 from math import sin, cos, tan, pi, ceil
@@ -25,9 +21,10 @@ from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase import pdfdoc
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfgen  import pdfgeom, pathobject, textobject
-from reportlab.lib.colors import Color, CMYKColor, toColor
-from reportlab.lib.utils import import_zlib
-from reportlab.lib.utils import fp_str
+from reportlab.lib.utils import import_zlib, ImageReader, fp_str
+from reportlab.lib.boxstuff import aspectRatioFix, anchorAdjustXY
+
+digitPat = re.compile('\d')  #used in decimal alignment
 
 zlib = import_zlib()
 _SeqTypes=(TupleType,ListType)
@@ -64,7 +61,25 @@ else:
     def _digester(s):
         return join(map(lambda x : "%02x" % ord(x), md5.md5(s).digest()), '')
 
-class Canvas:
+def _annFormat(D,color,thickness,dashArray,hradius=0,vradius=0):
+    from reportlab.pdfbase.pdfdoc import PDFArray, PDFDictionary
+    if color and not D.has_key('C'):
+        D["C"] = PDFArray([color.red, color.green, color.blue])
+    if not D.has_key('Border'):
+        border = [hradius,vradius,thickness or 0]
+        if dashArray:
+            border.append(PDFArray(dashArray))
+        D["Border"] = PDFArray(border)
+#   BS = PDFDictionary()
+#   bss = 'S'
+#   if dashArray:
+#       BS['D'] = PDFArray(dashArray)
+#       bss = 'D'
+#   BS['W'] = thickness or 0
+#   BS['S'] = bss
+#   D['BS'] = BS
+
+class Canvas(textobject._PDFColorSetter):
     """This class is the programmer's interface to the PDF file format.  Methods
     are (or will be) provided here to do just about everything PDF can do.
 
@@ -115,20 +130,21 @@ class Canvas:
                  pagesize=None,
                  bottomup = 1,
                  pageCompression=None,
-                 encoding = None,
                  invariant = None,
                  verbosity=0):
         """Create a canvas of a given size. etc.
+
+        You may pass a file-like object to filename as an alternative to
+        a string.
+        
         Most of the attributes are private - we will use set/get methods
         as the preferred interface.  Default page size is A4."""
         if pagesize is None: pagesize = rl_config.defaultPageSize
-        if encoding is None: encoding = rl_config.defaultEncoding
         if invariant is None: invariant = rl_config.invariant
         self._filename = filename
-        self._encodingName = encoding
-        self._doc = pdfdoc.PDFDocument(encoding,
-                                       compression=pageCompression,
-                                       invariant=invariant)
+
+        self._doc = pdfdoc.PDFDocument(compression=pageCompression,
+                                       invariant=invariant, filename=filename)
 
 
         #this only controls whether it prints 'saved ...' - 0 disables
@@ -153,6 +169,8 @@ class Canvas:
         self._annotationCount = 0
 
         self._outlines = [] # list for a name tree
+        self._psCommandsBeforePage = [] #for postscript tray/font commands
+        self._psCommandsAfterPage = [] #for postscript tray/font commands
 
         #PostScript has the origin at bottom left. It is easy to achieve a top-
         #down coord system by translating to the top of the page and setting y
@@ -161,17 +179,17 @@ class Canvas:
         #drawing coordinates.
         self.bottomup = bottomup
         self.imageCaching = rl_config.defaultImageCaching
-        self._make_preamble()
         self.init_graphics_state()
+        self._make_preamble()
         self.state_stack = []
 
     def init_graphics_state(self):
         #initial graphics state, never modify any of these in place
         self._x = 0
         self._y = 0
-        self._fontname = 'Times-Roman'
+        self._fontname = 'Helvetica'
         self._fontsize = 12
-        self._dynamicFont = 0
+
         self._textMode = 0  #track if between BT/ET
         self._leading = 14.4
         self._currentMatrix = (1., 0., 0., 1., 0., 0.)
@@ -197,42 +215,20 @@ class Canvas:
         self._strokeColorRGB = (0,0,0)
 
     def push_state_stack(self):
-
         state = {}
         d = self.__dict__
         for name in self.STATE_ATTRIBUTES:
             state[name] = d[name] #getattr(self, name)
-        #state = d.copy() # this is no faster.
-        #del state["state_stack"]
-        self.state_stack.append(state)
-        return
-
-        stateatts = self.STATE_ATTRIBUTES # list based alternative (not used)
-        staterange = self.STATE_RANGE
-        state = list(staterange)
-        d = self.__dict__
-        for i in staterange:
-            state[i] = d[stateatts[i]] #getattr(self, stateatts[i])
         self.state_stack.append(state)
 
     def pop_state_stack(self):
-
         state = self.state_stack[-1]
         del self.state_stack[-1]
         d = self.__dict__
-        #for name in self.STATE_ATTRIBUTES:
-        #    d[name] = state[name] #setattr(self, name, state[name])
         d.update(state)
-        return
-
-        stateatts = self.STATE_ATTRIBUTES # list based alternative (not used)
-        staterange = self.STATE_RANGE
-        d = self.__dict__
-        for i in staterange:
-            d[i] = stateatts[i] #setattr(self, stateatts[i], state[i])
 
     STATE_ATTRIBUTES = split("""
-     _x _y _fontname _fontsize _dynamicFont _textMode _leading _currentMatrix _fillMode
+     _x _y _fontname _fontsize _textMode _leading _currentMatrix _fillMode
      _fillMode _charSpace _wordSpace _horizScale _textRenderMode _rise _textLineMatrix
      _textMatrix _lineCap _lineJoin _lineDash _lineWidth _mitreLimit _fillColorRGB
      _strokeColorRGB""")
@@ -242,7 +238,7 @@ class Canvas:
 
     def _make_preamble(self):
         # yuk
-        iName = self._doc.getInternalFontName('Helvetica')
+        iName = self._doc.getInternalFontName(self._fontname)
         if self.bottomup:
             #must set an initial font
             self._preamble = '1 0 0 1 0 0 cm BT %s 12 Tf 14.4 TL ET' % iName
@@ -258,9 +254,14 @@ class Canvas:
     def setAuthor(self, author):
         """identify the author for invisible embedding inside the PDF document.
            the author annotation will appear in the the text of the file but will
-           not automatically be seen when the document is viewed."""
+           not automatically be seen when the document is viewed, but is visible
+           in document properties etc etc."""
         self._doc.setAuthor(author)
 
+    def setDateFormatter(self, dateFormatter):
+        """accepts a func(yyyy,mm,dd,hh,m,s) used to create embedded formatted date"""
+        self._doc.setDateFormatter(dateFormatter)
+
     def addOutlineEntry(self, title, key, level=0, closed=None):
         """Adds a new entry to the outline at given level.  If LEVEL not specified,
         entry goes at the top level.  If level specified, it must be
@@ -328,8 +329,7 @@ class Canvas:
           each of the string names inside must be bound to a bookmark
           before the document is generated.
         """
-        #print nametree
-        apply(self._doc.outline.setNames, (self,)+nametree)
+        self._doc.outline.setNames(*((self,)+nametree))
 
     def setTitle(self, title):
         """write a title into the PDF file that won't automatically display
@@ -341,6 +341,14 @@ class Canvas:
            in the document itself."""
         self._doc.setSubject(subject)
 
+    def setKeywords(self, keywords):
+        """write a list of keywords into the PDF file which shows in document properties.
+        Either submit a single string or a list/tuple"""
+        if type(keywords) in (TupleType, ListType):
+            keywords = ', '.join(keywords)
+        self._doc.setKeywords(keywords)
+        
+
     def pageHasData(self):
         "Info function - app can call it after showPage to see if it needs a save"
         return len(self._code) == 0
@@ -359,6 +367,7 @@ class Canvas:
 
     def showPage(self):
         """Close the current page and possibly start on a new page."""
+
         # ensure a space at the end of the stream - Acrobat does
         # not mind, but Ghostscript dislikes 'Qendstream' even if
         # the length marker finishes after 'Q'
@@ -372,8 +381,9 @@ class Canvas:
         page.setCompression(self._pageCompression)
         if self._pageDuration is not None:
             page.Dur = self._pageDuration
-        #print stream
-        page.setStream([self._preamble] + self._code)
+
+        strm =  self._psCommandsBeforePage + [self._preamble] + self._code + self._psCommandsAfterPage
+        page.setStream(strm)
         self._setXObjects(page)
         self._setAnnotations(page)
         self._doc.addPage(page)
@@ -417,7 +427,7 @@ class Canvas:
         return result
 
     def bookmarkPage(self, key,
-                      fitType="Fit",
+                      fit="Fit",
                       left=None,
                       top=None,
                       bottom=None,
@@ -469,30 +479,30 @@ class Canvas:
         if zoom is None:
             zoom = "null"
 
-        if fitType == "XYZ":
+        if fit == "XYZ":
             dest.xyz(left,top,zoom)
-        elif fitType == "Fit":
+        elif fit == "Fit":
             dest.fit()
-        elif fitType == "FitH":
+        elif fit == "FitH":
             dest.fith(top)
-        elif fitType == "FitV":
+        elif fit == "FitV":
             dest.fitv(left)
-        elif fitType == "FitR":
+        elif fit == "FitR":
             dest.fitr(left,bottom,right,top)
         #Do we need these (version 1.1 / Acrobat 3 versions)?
-        elif fitType == "FitB":
+        elif fit == "FitB":
             dest.fitb()
-        elif fitType == "FitBH":
+        elif fit == "FitBH":
             dest.fitbh(top)
-        elif fitType == "FitBV":
+        elif fit == "FitBV":
             dest.fitbv(left)
         else:
-            raise "Unknown Fit type %s" % (fitType,)
+            raise "Unknown Fit type %s" % (fit,)
 
         dest.setPage(pageref)
         return dest
 
-    def bookmarkHorizontalAbsolute(self, key, yhorizontal):
+    def bookmarkHorizontalAbsolute(self, key, top, left=0, fit='XYZ', **kw):
         """Bind a bookmark (destination) to the current page at a horizontal position.
            Note that the yhorizontal of the book mark is with respect to the default
            user space (where the origin is at the lower left corner of the page)
@@ -501,12 +511,12 @@ class Canvas:
            responsible for making sure the bookmark matches an appropriate item on
            the page."""
         #This method should probably be deprecated since it is just a sub-set of bookmarkPage
-        return self.bookmarkPage(key,fitType="FitH",top=yhorizontal)
+        return self.bookmarkPage(key, fit=fit, top=top, left=left, zoom=0)
 
-    def bookmarkHorizontal(self, key, relativeX, relativeY):
+    def bookmarkHorizontal(self, key, relativeX, relativeY, **kw):
         """w.r.t. the current transformation, bookmark this horizontal."""
-        (xt, yt) = self.absolutePosition(relativeX,relativeY)
-        self.bookmarkHorizontalAbsolute(key, yt)
+        (left, top) = self.absolutePosition(relativeX,relativeY)
+        self.bookmarkHorizontalAbsolute(key, top, left=left, **kw)
 
     #def _inPage0(self):  disallowed!
     #    """declare a page, enable page features"""
@@ -538,22 +548,36 @@ class Canvas:
         #
         ######################################################
 
-    def drawInlineImage(self, image, x,y, width=None,height=None):
+    def drawInlineImage(self, image, x,y, width=None,height=None,preserveAspectRatio=False,anchor='sw'):
         """Draw an Image into the specified rectangle.  If width and
         height are omitted, they are calculated from the image size.
         Also allow file names as well as images.  The size in pixels
-        of the image is returned."""
+        of the image is returned.
+
+        New post version 2.0:  drawImage can center the image in a box you provide.
+        This saves developers from doing lots of math in their code.  You can
+        define a box with x,y,width and height.   If 'preserveAspectRatio'
+        is set, the image will be scaled up or down as needed in proportion to
+        fit entirely within the box.  Unless the aspect ratio perfectly matches
+        the box, this will leave some space.  So, the boxAnchor property defines
+        how the image should be anchored in the box, using imaginary points of
+        the compass.  'sw' for SouthWest is the default, but 'c' for center
+        will center it in the given box.
+
+
+
+        """
 
         self._currentPageHasImages = 1
         from pdfimages import PDFImage
         img_obj = PDFImage(image, x,y, width, height)
-        img_obj.drawInlineImage(self)
+        img_obj.drawInlineImage(self,anchor=anchor,preserveAspectRatio=preserveAspectRatio)
         return (img_obj.width, img_obj.height)
 
-    def drawImage(self, image, x, y, width=None, height=None, mask=None):
+    def drawImage(self, image, x, y, width=None, height=None, mask=None, preserveAspectRatio=False, anchor='sw'):
         """Draws the image (ImageReader object or filename) as specified.
 
-        "image" may be an image filename or a ImageReader object.  If width
+        "image" may be an image filename or an ImageReader object.  If width
         and height are not given, the "natural" width and height in pixels
         is used at a scale of 1 point to 1 pixel.
 
@@ -566,6 +590,16 @@ class Canvas:
         The method returns the width and height of the underlying image since
         this is often useful for layout algorithms.
 
+        New post version 2.0:  drawImage can center the image in a box you provide.
+        This saves developers from doing lots of math in their code.  You can
+        define a box with x,y,width and height.   If 'preserveAspectRatio'
+        is set, the image will be scaled up or down as needed in proportion to
+        fit entirely within the box.  Unless the aspect ratio perfectly matches
+        the box, this will leave some space.  So, the boxAnchor property defines
+        how the image should be anchored in the box, using imaginary points of
+        the compass.  'sw' for SouthWest is the default, but 'c' for center
+        will center it in the given box.
+
         Unlike drawInlineImage, this creates 'external images' which
         are only stored once in the PDF file but can be drawn many times.
         If you give it the same filename twice, even at different locations
@@ -579,12 +613,12 @@ class Canvas:
 
         # first, generate a unique name/signature for the image.  If ANYTHING
         # is different, even the mask, this should be different.
-        if type(image) == type(''):
+        if isinstance(image,ImageReader):
+            rawdata = image.getRGBData()
+            name = _digester(rawdata+str(mask))
+        else:
             #filename, use it
             name = _digester('%s%s' % (image, mask))
-        else:
-            rawdata = image.getRGBData()
-            name = _digester(rawdata)
 
         # in the pdf document, this will be prefixed with something to
         # say it is an XObject.  Does it exist yet?
@@ -599,10 +633,8 @@ class Canvas:
             self._doc.addForm(name, imgObj)
 
         # ensure we have a size, as PDF will make it 1x1 pixel otherwise!
-        if width is None:
-            width = imgObj.width
-        if height is None:
-            height = imgObj.height
+        x,y,width,height,scaled = aspectRatioFix(preserveAspectRatio,anchor,x,y,width,height,imgObj.width,imgObj.height)
+        x,y = anchorAdjustXY(anchor,x,y,width,height)
 
         # scale and draw
         self.saveState()
@@ -624,6 +656,7 @@ class Canvas:
             (self._code, self._formsinuse, self._annotationrefs, self._formData) = saved
         else:
             self._code = []    # ready for more...
+            self._psCommandsAfterPage = []
             self._currentPageHasImages = 1 # for safety...
             self._formsinuse = []
             self._annotationrefs = []
@@ -679,9 +712,14 @@ class Canvas:
         self.pop_state_stack()
 
 
-    def addPostScriptCommand(self, command):
+    def addPostScriptCommand(self, command, position=1):
         """Embed literal Postscript in the document.
 
+        With position=0, it goes at very beginning of page stream;
+        with position=1, at current point; and
+        with position=2, at very end of page stream.  What that does
+        to the resulting Postscript depends on Adobe's header :-)
+
         Use with extreme caution, but sometimes needed for printer tray commands.
         Acrobat 4.0 will export Postscript to a printer or file containing
         the given commands.  Adobe Reader 6.0 no longer does as this feature is
@@ -701,32 +739,55 @@ class Canvas:
             self._setXObjects(psObj)
             self._doc.Reference(psObj, regName)
             self._doc.addForm(rawName, psObj)
-        self._code.append("/%s Do" % regName)
+        if position == 0:
+            self._psCommandsBeforePage.append("/%s Do" % regName)
+        elif position==1:
+            self._code.append("/%s Do" % regName)
+        else:
+            self._psCommandsAfterPage.append("/%s Do" % regName)
+
         self._formsinuse.append(rawName)
 
+    def _absRect(self,rect,relative=0):
+        if not rect:
+            w,h = self._pagesize
+            rect = (0,0,w,h)
+        elif relative:
+            lx, ly, ux, uy = rect
+            xll,yll = self.absolutePosition(lx,ly)
+            xur,yur = self.absolutePosition(ux, uy)
+            xul,yul = self.absolutePosition(lx, uy)
+            xlr,ylr = self.absolutePosition(ux, ly)
+            xs = xll, xur, xul, xlr
+            ys = yll, yur, yul, ylr
+            xmin, ymin = min(xs), min(ys)
+            xmax, ymax = max(xs), max(ys)
+            rect = xmin, ymin, xmax, ymax
+        return rect
 
-        #
+    def freeTextAnnotation(self, contents, DA, Rect=None, addtopage=1, name=None, relative=0, **kw):
+        """DA is the default appearance string???"""
+        Rect = self._absRect(Rect,relative)
+        self._addAnnotation(pdfdoc.FreeTextAnnotation(Rect, contents, DA, **kw), name, addtopage)
 
-    def textAnnotation0(self, contents, Rect=None, addtopage=1, name=None, **kw):
-        """Experimental.
+    def textAnnotation(self, contents, Rect=None, addtopage=1, name=None, relative=0, **kw):
+        """Experimental, but works.
         """
-        if not Rect:
-            (w,h) = self._pagesize# default to whole page (?)
-            Rect = (0,0,w,h)
-        annotation = apply(pdfdoc.TextAnnotation, (Rect, contents), kw)
-        self._addAnnotation(annotation, name, addtopage)
+        Rect = self._absRect(Rect,relative)
+        self._addAnnotation(pdfdoc.TextAnnotation(Rect, contents, **kw), name, addtopage)
+    textAnnotation0 = textAnnotation    #deprecated
 
-    def inkAnnotation0(self, contents, InkList=None, Rect=None, addtopage=1, name=None, **kw):
+    def inkAnnotation(self, contents, InkList=None, Rect=None, addtopage=1, name=None, relative=0, **kw):
+        raise NotImplementedError
         "Experimental"
-        (w,h) = self._pagesize
-        if not Rect:
-            Rect = (0,0,w,h)
+        Rect = self._absRect(Rect,relative)
         if not InkList:
-            InkList = ( (100,100,100,h-100,w-100,h-100,w-100,100), )
-        annotation = apply(pdfdoc.InkAnnotation, (Rect, contents, InkList), kw)
-        self.addAnnotation(annotation, name, addtopage)
+            InkList = ((100,100,100,h-100,w-100,h-100,w-100,100),)
+        self._addAnnotation(pdfdoc.InkAnnotation(Rect, contents, InkList, **kw), name, addtopage)
+    inkAnnotation0 = inkAnnotation  #deprecated
 
-    def linkAbsolute(self, contents, destinationname, Rect=None, addtopage=1, name=None, **kw):
+    def linkAbsolute(self, contents, destinationname, Rect=None, addtopage=1, name=None,
+            thickness=0, color=None, dashArray=None, **kw):
         """rectangular link annotation positioned wrt the default user space.
            The identified rectangle on the page becomes a "hot link" which
            when clicked will send the viewer to the page and position identified
@@ -741,39 +802,23 @@ class Canvas:
 
            You may want to use the keyword argument Border='[0 0 0]' to
            suppress the visible rectangle around the during viewing link."""
-        destination = self._bookmarkReference(destinationname) # permitted to be undefined... must bind later...
-        (w,h) = self._pagesize
-        if not Rect:
-            Rect = (0,0,w,h)
-        kw["Rect"] = Rect
-        kw["Contents"] = contents
-        kw["Destination"] = destination
-        annotation = apply(pdfdoc.LinkAnnotation, (), kw)
-        self._addAnnotation(annotation, name, addtopage)
+        return self.linkRect(contents, destinationname, Rect, addtopage, name, relative=0,
+                thickness=thickness, color=color, dashArray=dashArray, **kw)
 
-    def linkRect(self, contents, destinationname, Rect=None, addtopage=1, name=None, **kw):
+    def linkRect(self, contents, destinationname, Rect=None, addtopage=1, name=None, relative=1,
+            thickness=0, color=None, dashArray=None, **kw):
         """rectangular link annotation w.r.t the current user transform.
            if the transform is skewed/rotated the absolute rectangle will use the max/min x/y
         """
-        if Rect is None:
-            pass # do whatever linkAbsolute does
-        else:
-            (lx, ly, ux, uy) = Rect
-            (xll,yll) = self.absolutePosition(lx,ly)
-            (xur,yur) = self.absolutePosition(ux, uy)
-            (xul,yul) = self.absolutePosition(lx, uy)
-            (xlr,ylr) = self.absolutePosition(ux, ly)
-            xs = (xll, xur, xul, xlr)
-            ys = (yll, yur, yul, ylr)
-            (xmin, ymin) = (min(xs), min(ys))
-            (xmax, ymax) = (max(xs), max(ys))
-            #(w2, h2) = (xmax-xmin, ymax-ymin)
-            Rect = (xmin, ymin, xmax, ymax)
-            #print "rect is", Rect
-        return apply(self.linkAbsolute, (contents, destinationname, Rect, addtopage, name), kw)
+        destination = self._bookmarkReference(destinationname) # permitted to be undefined... must bind later...
+        Rect = self._absRect(Rect,relative)
+        kw["Rect"] = Rect
+        kw["Contents"] = contents
+        kw["Destination"] = destination
+        _annFormat(kw,color,thickness,dashArray)
+        return self._addAnnotation(pdfdoc.LinkAnnotation(**kw), name, addtopage)
 
-
-    def linkURL(self, url, rect, relative=0, thickness=0, color=None, dashArray=None):
+    def linkURL(self, url, rect, relative=0, thickness=0, color=None, dashArray=None, kind="URI", **kw):
         """Create a rectangular URL 'hotspot' in the given rectangle.
 
         if relative=1, this is in the current coord system, otherwise
@@ -786,42 +831,27 @@ class Canvas:
         from reportlab.pdfbase.pdfdoc import PDFDictionary, PDFName, PDFArray, PDFString
         #tried the documented BS element in the pdf spec but it
         #does not work, and Acrobat itself does not appear to use it!
-        if relative:
-            #adjust coordinates
-            (lx, ly, ux, uy) = rect
-            (xll,yll) = self.absolutePosition(lx,ly)
-            (xur,yur) = self.absolutePosition(ux, uy)
-            (xul,yul) = self.absolutePosition(lx, uy)
-            (xlr,ylr) = self.absolutePosition(ux, ly)
-            xs = (xll, xur, xul, xlr)
-            ys = (yll, yur, yul, ylr)
-            (xmin, ymin) = (min(xs), min(ys))
-            (xmax, ymax) = (max(xs), max(ys))
-            #(w2, h2) = (xmax-xmin, ymax-ymin)
-            rect = (xmin, ymin, xmax, ymax)
 
-        ann = PDFDictionary()
+        ann = PDFDictionary(dict=kw)
         ann["Type"] = PDFName("Annot")
         ann["Subtype"] = PDFName("Link")
-        ann["Rect"] = PDFArray(rect) # the whole page for testing
+        ann["Rect"] = PDFArray(self._absRect(rect,relative)) # the whole page for testing
 
         # the action is a separate dictionary
         A = PDFDictionary()
         A["Type"] = PDFName("Action") # not needed?
-        A["S"] = PDFName("URI")
-        A["URI"] = PDFString(url)
+        uri = PDFString(url)
+        A['S'] = PDFName(kind)
+        if kind=="URI":
+            A["URI"] = uri
+        elif kind=='GoToR':
+            A["F"] = uri
+            A["D"] = "[ 0 /XYZ null null null ]"
+        else:
+            raise ValueError("Unknown linkURI kind '%s'" % kind)
+
         ann["A"] = A
-
-        # now for formatting stuff.
-        if color:
-            ann["C"] = PDFArray([color.red, color.green, color.blue])
-        border = [0,0,0]
-        if thickness:
-            border[2] = thickness
-        if dashArray:
-            border.append(PDFArray(dashArray))
-        ann["Border"] = PDFArray(border)
-
+        _annFormat(ann,color,thickness,dashArray)
         self._addAnnotation(ann)
 
     def _addAnnotation(self, annotation, name=None, addtopage=1):
@@ -895,7 +925,6 @@ class Canvas:
         resultb = -selfb/det
         resultf = (selfe*selfb - selff*selfa)/det
         self.transform(resulta, resultb, resultc, resultd, resulte, resultf)
-        #print self._currentMatrix
 
     def transform(self, a,b,c,d,e,f):
         """adjoin a mathematical transform to the current graphics state matrix.
@@ -906,8 +935,6 @@ class Canvas:
             self._currentMatrix = (a0*a+c0*b,    b0*a+d0*b,
                                    a0*c+c0*d,    b0*c+d0*d,
                                    a0*e+c0*f+e0, b0*e+d0*f+f0)
-        #print "transform", (a,b,c,d,e,f)
-        #print "currentMatrix", self._currentMatrix
         if self._code and self._code[-1][-3:]==' cm':
             L = split(self._code[-1])
             a0, b0, c0, d0, e0, f0 = map(float,L[-7:-1])
@@ -975,13 +1002,11 @@ class Canvas:
             canvas.restoreState()
             # if the save/restore pairs match then font is Helvetica 20 again.
         """
-        #"""These need expanding to save/restore Python's state tracking too"""
         self.push_state_stack()
         self._code.append('q')
 
     def restoreState(self):
         """restore the graphics state to the matching saved state (see saveState)."""
-        #"""These need expanding to save/restore Python's state tracking too"""
         self._code.append('Q')
         self.pop_state_stack()
 
@@ -1169,20 +1194,6 @@ class Canvas:
         # use PDFTextObject for multi-line text.
         ##################################################
 
-    def setFillColorCMYK(self, c, m, y, k):
-         """set the fill color useing negative color values
-            (cyan, magenta, yellow and darkness value).
-         Takes 4 arguments between 0.0 and 1.0"""
-         self._fillColorCMYK = (c, m, y, k)
-         self._code.append('%s k' % fp_str(c, m, y, k))
-
-    def setStrokeColorCMYK(self, c, m, y, k):
-         """set the stroke color useing negative color values
-            (cyan, magenta, yellow and darkness value).
-            Takes 4 arguments between 0.0 and 1.0"""
-         self._strokeColorCMYK = (c, m, y, k)
-         self._code.append('%s K' % fp_str(c, m, y, k))
-
     def drawString(self, x, y, text):
         """Draws a string in the current text styles."""
         #we could inline this for speed if needed
@@ -1204,6 +1215,56 @@ class Canvas:
         t.textLine(text)
         self.drawText(t)
 
+    def drawAlignedString(self, x, y, text, pivotChar="."):
+        """Draws a string aligned on the first '.' (or other pivot character).
+
+        The centre position of the pivot character will be used as x.
+        So, you could draw a straight line down through all the decimals in a
+        column of numbers, and anything without a decimal should be
+        optically aligned with those that have.
+
+        There is one special rule to help with accounting formatting.  Here's
+        how normal numbers should be aligned on the 'dot'. Look at the
+        LAST two:
+           12,345,67
+              987.15
+               42
+           -1,234.56
+             (456.78)
+             (456)
+               27 inches
+               13cm
+        Since the last three do not contain a dot, a crude dot-finding
+        rule would place them wrong. So we test for the special case
+        where no pivot is found, digits are present, but the last character
+        is not a digit.  We then work back from the end of the string
+        This case is a tad slower but hopefully rare.
+        
+        """
+        parts = text.split(pivotChar,1)
+        pivW = self.stringWidth(pivotChar, self._fontname, self._fontsize)
+        
+        if len(parts) == 1 and digitPat.search(text) is not None and text[-1] not in digits:
+            #we have no decimal but it ends in a bracket, or 'in' or something.
+            #the cut should be after the last digit.
+            leftText = parts[0][0:-1]
+            rightText = parts[0][-1]
+            #any more?
+            while leftText[-1] not in digits:
+                rightText = leftText[-1] + rightText
+                leftText = leftText[0:-1]
+
+            self.drawRightString(x-0.5*pivW, y, leftText)
+            self.drawString(x-0.5*pivW, y, rightText)
+
+        else:
+            #normal case
+            leftText = parts[0]
+            self.drawRightString(x-0.5*pivW, y, leftText)
+            if len(parts) > 1:
+                rightText = pivotChar + parts[1]
+                self.drawString(x-0.5*pivW, y, rightText)
+
     def getAvailableFonts(self):
         """Returns the list of PostScript font names available.
 
@@ -1233,25 +1294,26 @@ class Canvas:
         font size. Raises a readable exception if an illegal font
         is supplied.  Font names are case-sensitive! Keeps track
         of font name and size for metrics."""
-        #print "setFont", (psfontname, size, leading)
         self._fontname = psfontname
         self._fontsize = size
         if leading is None:
             leading = size * 1.2
         self._leading = leading
         font = pdfmetrics.getFont(self._fontname)
-        self._dynamicFont = getattr(font, '_dynamicFont', 0)
-        if not self._dynamicFont:
+        if not font._dynamicFont:
             pdffontname = self._doc.getInternalFontName(psfontname)
             self._code.append('BT %s %s Tf %s TL ET' % (pdffontname, fp_str(size), fp_str(leading)))
 
-    def stringWidth(self, text, fontName, fontSize, encoding=None):
+    def setFontSize(self, size=None, leading=None):
+        '''Sets font size or leading without knowing the font face'''
+        if size is None: size = self._fontsize
+        if leading is None: leading = self._leading
+        self.setFont(self._fontname, size, leading)
+
+    def stringWidth(self, text, fontName=None, fontSize=None):
         "gets width of a string in the given font and size"
-        if encoding is not None:
-            import logger
-            logger.warnOnce('encoding argument to Canvas.stringWidth is deprecated and has no effect!')
-        #if encoding is None: encoding = self._doc.encoding
-        return pdfmetrics.stringWidth(text, fontName, fontSize)
+        return pdfmetrics.stringWidth(text, fontName or self._fontname,
+                                    (fontSize,self._fontsize)[fontSize is None])
 
     # basic graphics modes
 
@@ -1281,82 +1343,9 @@ class Canvas:
             self._code.append('[%s %s] 0 d' % (array, phase))
         elif type(array) == ListType or type(array) == TupleType:
             assert phase >= 0, "phase is a length in user space"
-            textarray = join(map(str, array))
+            textarray = ' '.join(map(str, array))
             self._code.append('[%s] %s d' % (textarray, phase))
 
-    def setFillColorRGB(self, r, g, b):
-        """Set the fill color using positive color description
-           (Red,Green,Blue).  Takes 3 arguments between 0.0 and 1.0"""
-        self._fillColorRGB = (r, g, b)
-        self._code.append('%s rg' % fp_str(r,g,b))
-
-    def setStrokeColorRGB(self, r, g, b):
-        """Set the stroke color using positive color description
-           (Red,Green,Blue).  Takes 3 arguments between 0.0 and 1.0"""
-        self._strokeColorRGB = (r, g, b)
-        self._code.append('%s RG' % fp_str(r,g,b))
-
-    def setFillColor(self, aColor):
-        """Takes a color object, allowing colors to be referred to by name"""
-        if isinstance(aColor, CMYKColor):
-            d = aColor.density
-            c,m,y,k = (d*aColor.cyan, d*aColor.magenta, d*aColor.yellow, d*aColor.black)
-            self._fillColorCMYK = (c, m, y, k)
-            self._code.append('%s k' % fp_str(c, m, y, k))
-        elif isinstance(aColor, Color):
-            rgb = (aColor.red, aColor.green, aColor.blue)
-            self._fillColorRGB = rgb
-            self._code.append('%s rg' % fp_str(rgb) )
-        elif type(aColor) in _SeqTypes:
-            l = len(aColor)
-            if l==3:
-                self._fillColorRGB = aColor
-                self._code.append('%s rg' % fp_str(aColor) )
-            elif l==4:
-                self.setFillColorCMYK(aColor[0], aColor[1], aColor[2], aColor[3])
-            else:
-                raise 'Unknown color', str(aColor)
-        elif type(aColor) is StringType:
-            self.setFillColor(toColor(aColor))
-        else:
-            raise 'Unknown color', str(aColor)
-
-    def setStrokeColor(self, aColor):
-        """Takes a color object, allowing colors to be referred to by name"""
-        if isinstance(aColor, CMYKColor):
-            d = aColor.density
-            c,m,y,k = (d*aColor.cyan, d*aColor.magenta, d*aColor.yellow, d*aColor.black)
-            self._strokeColorCMYK = (c, m, y, k)
-            self._code.append('%s K' % fp_str(c, m, y, k))
-        elif isinstance(aColor, Color):
-        #if type(aColor) == ColorType:
-            rgb = (aColor.red, aColor.green, aColor.blue)
-            self._strokeColorRGB = rgb
-            self._code.append('%s RG' % fp_str(rgb) )
-        elif type(aColor) in _SeqTypes:
-            l = len(aColor)
-            if l==3:
-                self._strokeColorRGB = aColor
-                self._code.append('%s RG' % fp_str(aColor) )
-            elif l==4:
-                self.setStrokeColorCMYK(aColor[0], aColor[1], aColor[2], aColor[3])
-            else:
-                raise 'Unknown color', str(aColor)
-        elif type(aColor) is StringType:
-            self.setStrokeColor(toColor(aColor))
-        else:
-            raise 'Unknown color', str(aColor)
-
-    def setFillGray(self, gray):
-        """Sets the gray level; 0.0=black, 1.0=white"""
-        self._fillColorRGB = (gray, gray, gray)
-        self._code.append('%s g' % fp_str(gray))
-
-    def setStrokeGray(self, gray):
-        """Sets the gray level; 0.0=black, 1.0=white"""
-        self._strokeColorRGB = (gray, gray, gray)
-        self._code.append('%s G' % fp_str(gray))
-
     # path stuff - the separate path object builds it
 
     def beginPath(self):
@@ -1489,10 +1478,18 @@ class Canvas:
             transDict[key] = value
         self._pageTransition = transDict
 
+    def getCurrentPageContent(self):
+        """Return uncompressed contents of current page buffer.
+
+        This is useful in creating test cases and assertions of what
+        got drawn, without necessarily saving pages to disk"""
+        return '\n'.join(self._code)
+
+
+
 if _instanceEscapePDF:
     import new
     Canvas._escape = new.instancemethod(_instanceEscapePDF,None,Canvas)
 
-
 if __name__ == '__main__':
     print 'For test scripts, look in reportlab/test'
